"""
    :codeauthor: Rahul Handay <rahulha@saltstack.com>
"""

import logging

import pytest

import salt.states.host as host
from tests.support.mock import MagicMock, call, patch

log = logging.getLogger(__name__)


hostname = "salt"
localhost_ip = "127.0.0.1"
ip_list = ["203.0.113.113", "203.0.113.14"]
default_hosts = {
    ip_list[0]: {"aliases": [hostname]},
    ip_list[1]: {"aliases": [hostname]},
}


@pytest.fixture
def configure_loader_modules():
    return {
        host: {"__opts__": {"test": False}},
    }


def test_present():
    """
    Test to ensures that the named host is present with the given ip
    """
    add_host = MagicMock(return_value=True)
    rm_host = MagicMock(return_value=True)
    hostname = "salt"
    ip_str = "127.0.0.1"
    ip_list = ["10.1.2.3", "10.4.5.6"]

    # Case 1: No match for hostname. Single IP address passed to the state.
    list_hosts = MagicMock(return_value={"127.0.0.1": {"aliases": ["localhost"]}})
    with patch.dict(
        host.__salt__,
        {
            "hosts.list_hosts": list_hosts,
            "hosts.add_host": add_host,
            "hosts.rm_host": rm_host,
        },
    ):
        ret = host.present(hostname, ip_str)
        assert ret["result"] is True
        assert ret["comment"] == "Added host {} ({})".format(hostname, ip_str), ret[
            "comment"
        ]
        assert ret["changes"] == {"added": {ip_str: [hostname]}}, ret["changes"]
        expected = [call(ip_str, hostname)]
        assert add_host.mock_calls == expected, add_host.mock_calls
        assert rm_host.mock_calls == [], rm_host.mock_calls

    # Case 2: No match for hostname. Multiple IP addresses passed to the
    # state.
    list_hosts = MagicMock(return_value={"127.0.0.1": {"aliases": ["localhost"]}})
    add_host.reset_mock()
    rm_host.reset_mock()
    with patch.dict(
        host.__salt__,
        {
            "hosts.list_hosts": list_hosts,
            "hosts.add_host": add_host,
            "hosts.rm_host": rm_host,
        },
    ):
        ret = host.present(hostname, ip_list)
        assert ret["result"] is True
        assert "Added host {} ({})".format(hostname, ip_list[0]) in ret["comment"]
        assert "Added host {} ({})".format(hostname, ip_list[1]) in ret["comment"]
        assert ret["changes"] == {
            "added": {ip_list[0]: [hostname], ip_list[1]: [hostname]}
        }, ret["changes"]
        expected = sorted(call(x, hostname) for x in ip_list)
        assert sorted(add_host.mock_calls) == expected, add_host.mock_calls
        assert rm_host.mock_calls == [], rm_host.mock_calls

    # Case 3: Match for hostname, but no matching IP. Single IP address
    # passed to the state.
    list_hosts = MagicMock(
        return_value={
            "127.0.0.1": {"aliases": ["localhost"]},
            ip_list[0]: {"aliases": [hostname]},
        }
    )
    add_host.reset_mock()
    rm_host.reset_mock()
    with patch.dict(
        host.__salt__,
        {
            "hosts.list_hosts": list_hosts,
            "hosts.add_host": add_host,
            "hosts.rm_host": rm_host,
        },
    ):
        ret = host.present(hostname, ip_str)
        assert ret["result"] is True
        assert "Added host {} ({})".format(hostname, ip_str) in ret["comment"]
        assert (
            "Host {} present for IP address {}".format(hostname, ip_list[0])
            in ret["warnings"][0]
        )
        assert ret["changes"] == {"added": {ip_str: [hostname]}}, ret["changes"]
        expected = [call(ip_str, hostname)]
        assert add_host.mock_calls == expected, add_host.mock_calls
        assert rm_host.mock_calls == [], rm_host.mock_calls

    # Case 3a: Repeat the above with clean=True
    add_host.reset_mock()
    rm_host.reset_mock()
    with patch.dict(
        host.__salt__,
        {
            "hosts.list_hosts": list_hosts,
            "hosts.add_host": add_host,
            "hosts.rm_host": rm_host,
        },
    ):
        ret = host.present(hostname, ip_str, clean=True)
        assert ret["result"] is True
        assert "Added host {} ({})".format(hostname, ip_str) in ret["comment"]
        assert "Removed host {} ({})".format(hostname, ip_list[0]) in ret["comment"]
        assert ret["changes"] == {
            "added": {ip_str: [hostname]},
            "removed": {ip_list[0]: [hostname]},
        }, ret["changes"]
        expected = [call(ip_str, hostname)]
        assert add_host.mock_calls == expected, add_host.mock_calls
        expected = [call(ip_list[0], hostname)]
        assert rm_host.mock_calls == expected, rm_host.mock_calls

    # Case 4: Match for hostname, but no matching IP. Multiple IP addresses
    # passed to the state.
    cur_ip = "1.2.3.4"
    list_hosts = MagicMock(
        return_value={
            "127.0.0.1": {"aliases": ["localhost"]},
            cur_ip: {"aliases": [hostname]},
        }
    )
    add_host.reset_mock()
    rm_host.reset_mock()
    with patch.dict(
        host.__salt__,
        {
            "hosts.list_hosts": list_hosts,
            "hosts.add_host": add_host,
            "hosts.rm_host": rm_host,
        },
    ):
        ret = host.present(hostname, ip_list)
        assert ret["result"] is True
        assert "Added host {} ({})".format(hostname, ip_list[0]) in ret["comment"]
        assert "Added host {} ({})".format(hostname, ip_list[1]) in ret["comment"]
        assert ret["changes"] == {
            "added": {ip_list[0]: [hostname], ip_list[1]: [hostname]},
        }, ret["changes"]
        expected = sorted(call(x, hostname) for x in ip_list)
        assert sorted(add_host.mock_calls) == expected, add_host.mock_calls
        assert rm_host.mock_calls == [], rm_host.mock_calls

    # Case 4a: Repeat the above with clean=True
    add_host.reset_mock()
    rm_host.reset_mock()
    with patch.dict(
        host.__salt__,
        {
            "hosts.list_hosts": list_hosts,
            "hosts.add_host": add_host,
            "hosts.rm_host": rm_host,
        },
    ):
        ret = host.present(hostname, ip_list, clean=True)
        assert ret["result"] is True
        assert "Added host {} ({})".format(hostname, ip_list[0]) in ret["comment"]
        assert "Added host {} ({})".format(hostname, ip_list[1]) in ret["comment"]
        assert "Removed host {} ({})".format(hostname, cur_ip) in ret["comment"]
        assert ret["changes"] == {
            "added": {ip_list[0]: [hostname], ip_list[1]: [hostname]},
            "removed": {cur_ip: [hostname]},
        }, ret["changes"]
        expected = sorted(call(x, hostname) for x in ip_list)
        assert sorted(add_host.mock_calls) == expected, add_host.mock_calls
        expected = [call(cur_ip, hostname)]
        assert rm_host.mock_calls == expected, rm_host.mock_calls

    # Case 5: Multiple IP addresses passed to the state. One of them
    # matches, the other does not. There is also a non-matching IP that
    # must be removed.
    cur_ip = "1.2.3.4"
    list_hosts = MagicMock(
        return_value={
            "127.0.0.1": {"aliases": ["localhost"]},
            cur_ip: {"aliases": [hostname]},
            ip_list[0]: {"aliases": [hostname]},
        }
    )
    add_host.reset_mock()
    rm_host.reset_mock()
    with patch.dict(
        host.__salt__,
        {
            "hosts.list_hosts": list_hosts,
            "hosts.add_host": add_host,
            "hosts.rm_host": rm_host,
        },
    ):
        ret = host.present(hostname, ip_list)
        assert ret["result"] is True
        assert "Added host {} ({})".format(hostname, ip_list[1]) in ret["comment"]
        assert ret["changes"] == {"added": {ip_list[1]: [hostname]}}, ret["changes"]
        expected = [call(ip_list[1], hostname)]
        assert add_host.mock_calls == expected, add_host.mock_calls
        assert rm_host.mock_calls == [], rm_host.mock_calls

    # Case 5a: Repeat the above with clean=True
    add_host.reset_mock()
    rm_host.reset_mock()
    with patch.dict(
        host.__salt__,
        {
            "hosts.list_hosts": list_hosts,
            "hosts.add_host": add_host,
            "hosts.rm_host": rm_host,
        },
    ):
        ret = host.present(hostname, ip_list, clean=True)
        assert ret["result"] is True
        assert "Added host {} ({})".format(hostname, ip_list[1]) in ret["comment"]
        assert "Removed host {} ({})".format(hostname, cur_ip) in ret["comment"]
        assert ret["changes"] == {
            "added": {ip_list[1]: [hostname]},
            "removed": {cur_ip: [hostname]},
        }, ret["changes"]
        expected = [call(ip_list[1], hostname)]
        assert add_host.mock_calls == expected, add_host.mock_calls
        expected = [call(cur_ip, hostname)]
        assert rm_host.mock_calls == expected, rm_host.mock_calls

    # Case 6: Single IP address passed to the state, which matches the
    # current configuration for that hostname. No changes should be made.
    list_hosts = MagicMock(return_value={ip_str: {"aliases": [hostname]}})
    add_host.reset_mock()
    rm_host.reset_mock()
    with patch.dict(
        host.__salt__,
        {
            "hosts.list_hosts": list_hosts,
            "hosts.add_host": add_host,
            "hosts.rm_host": rm_host,
        },
    ):
        ret = host.present(hostname, ip_str)
        assert ret["result"] is True
        assert (
            ret["comment"]
            == "Host {} ({}) already present".format(hostname, ip_str)
            in ret["comment"]
        )
        assert ret["changes"] == {}, ret["changes"]
        assert add_host.mock_calls == [], add_host.mock_calls
        assert rm_host.mock_calls == [], rm_host.mock_calls

    # Case 7: Multiple IP addresses passed to the state, which both match
    # the current configuration for that hostname. No changes should be
    # made.
    list_hosts = MagicMock(
        return_value={
            ip_list[0]: {"aliases": [hostname]},
            ip_list[1]: {"aliases": [hostname]},
        }
    )
    add_host.reset_mock()
    rm_host.reset_mock()
    with patch.dict(
        host.__salt__,
        {
            "hosts.list_hosts": list_hosts,
            "hosts.add_host": add_host,
            "hosts.rm_host": rm_host,
        },
    ):
        ret = host.present(hostname, ip_list)
        assert ret["result"] is True
        assert (
            "Host {} ({}) already present".format(hostname, ip_list[0])
            in ret["comment"]
        )
        assert (
            "Host {} ({}) already present".format(hostname, ip_list[1])
            in ret["comment"]
        )
        assert ret["changes"] == {}, ret["changes"]
        assert add_host.mock_calls == [], add_host.mock_calls
        assert rm_host.mock_calls == [], rm_host.mock_calls

    # Case 8: Passing a comment to host.present with multiple IPs
    list_hosts = MagicMock(return_value={"127.0.0.1": {"aliases": ["localhost"]}})
    add_host_mock = MagicMock(return_value=True)
    rm_host_mock = MagicMock(return_value=True)
    set_comment_mock = MagicMock(return_value=True)
    add_host_mock.reset_mock()
    rm_host_mock.reset_mock()
    set_comment_mock.reset_mock()
    with patch.dict(
        host.__salt__,
        {
            "hosts.list_hosts": list_hosts,
            "hosts.add_host": add_host_mock,
            "hosts.rm_host": rm_host_mock,
            "hosts.set_comment": set_comment_mock,
        },
    ):
        ret = host.present(hostname, ip_list, comment="A comment")
        assert ret["result"] is True
        assert "Added host {} ({})".format(hostname, ip_list[0]) in ret["comment"]
        assert "Added host {} ({})".format(hostname, ip_list[1]) in ret["comment"]
        assert (
            "Set comment for host {} (A comment)".format(ip_list[0]) in ret["comment"]
        )
        assert (
            "Set comment for host {} (A comment)".format(ip_list[1]) in ret["comment"]
        )
        assert ret["changes"] == {
            "added": {ip_list[0]: [hostname], ip_list[1]: [hostname]},
            "comment_added": {ip_list[0]: ["A comment"], ip_list[1]: ["A comment"]},
        }, ret["changes"]
        expected = sorted(call(x, hostname) for x in ip_list)
        assert sorted(add_host_mock.mock_calls) == expected, add_host_mock.mock_calls

        expected = sorted(call(x, "A comment") for x in ip_list)
        assert (
            sorted(set_comment_mock.mock_calls) == expected
        ), set_comment_mock.mock_calls

        assert rm_host_mock.mock_calls == [], rm_host_mock.mock_calls


def test_host_present_should_return_True_if_test_and_no_changes():
    add_host_mock = MagicMock(return_value=True)
    rm_host_mock = MagicMock(return_value=True)
    expected = {
        "comment": "Host {} ({}) already present".format(
            hostname,
            ip_list[0],
        ),
        "changes": {},
        "name": hostname,
        "result": True,
    }
    list_hosts = MagicMock(
        return_value={ip_list[0]: {"aliases": [hostname]}},
    )
    with patch.dict(
        host.__salt__,
        {
            "hosts.list_hosts": list_hosts,
            "hosts.add_host": add_host_mock,
            "hosts.rm_host": rm_host_mock,
        },
    ):
        with patch.dict(host.__opts__, {"test": True}):
            ret = host.present(hostname, ip_list[:1])

            assert ret == expected


def test_host_present_should_return_None_if_test_and_adding():
    add_host_mock = MagicMock(return_value=True)
    rm_host_mock = MagicMock(return_value=True)
    expected = {
        "comment": "\n".join(
            ["Host {} ({}) already present", "Host {} ({}) would be added"]
        ).format(
            hostname,
            ip_list[0],
            hostname,
            ip_list[1],
        ),
        "changes": {"added": {ip_list[1]: [hostname]}},
        "name": hostname,
        "result": None,
    }
    list_hosts = MagicMock(
        return_value={ip_list[0]: {"aliases": [hostname]}},
    )
    with patch.dict(
        host.__salt__,
        {
            "hosts.list_hosts": list_hosts,
            "hosts.add_host": add_host_mock,
            "hosts.rm_host": rm_host_mock,
        },
    ):
        with patch.dict(host.__opts__, {"test": True}):
            ret = host.present(hostname, ip_list)
            assert ret == expected


def test_host_present_should_return_None_if_test_and_removing():
    add_host_mock = MagicMock(return_value=True)
    rm_host_mock = MagicMock(return_value=True)
    list_hosts_mock = MagicMock(return_value=default_hosts)
    expected = {
        "comment": "\n".join(
            ["Host {} ({}) already present", "Host {} ({}) would be removed"]
        ).format(
            hostname,
            ip_list[0],
            hostname,
            ip_list[1],
        ),
        "changes": {"removed": {ip_list[1]: [hostname]}},
        "name": hostname,
        "result": None,
    }
    with patch.dict(
        host.__salt__,
        {
            "hosts.list_hosts": list_hosts_mock,
            "hosts.add_host": add_host_mock,
            "hosts.rm_host": rm_host_mock,
        },
    ):
        with patch.dict(host.__opts__, {"test": True}):
            ret = host.present(hostname, ip_list[:1], clean=True)
            assert ret == expected


def test_absent():
    """
    Test to ensure that the named host is absent
    """
    ret = {
        "changes": {},
        "comment": "Host salt (127.0.0.1) already absent",
        "name": "salt",
        "result": True,
    }

    mock = MagicMock(return_value=False)
    with patch.dict(host.__salt__, {"hosts.has_pair": mock}):
        assert host.absent("salt", "127.0.0.1") == ret


def test_only_already():
    """
    Test only() when the state hasn't changed
    """
    expected = {
        "name": "127.0.1.1",
        "changes": {},
        "result": True,
        "comment": 'IP address 127.0.1.1 already set to "foo.bar"',
    }
    mock1 = MagicMock(return_value=["foo.bar"])
    with patch.dict(host.__salt__, {"hosts.get_alias": mock1}):
        mock2 = MagicMock(return_value=True)
        with patch.dict(host.__salt__, {"hosts.set_host": mock2}):
            with patch.dict(host.__opts__, {"test": False}):
                assert expected == host.only("127.0.1.1", "foo.bar")


def test_only_dryrun():
    """
    Test only() when state would change, but it's a dry run
    """
    expected = {
        "name": "127.0.1.1",
        "changes": {},
        "result": None,
        "comment": 'Would change 127.0.1.1 from "foo.bar" to "foo.bar foo"',
    }
    mock1 = MagicMock(return_value=["foo.bar"])
    with patch.dict(host.__salt__, {"hosts.get_alias": mock1}):
        mock2 = MagicMock(return_value=True)
        with patch.dict(host.__salt__, {"hosts.set_host": mock2}):
            with patch.dict(host.__opts__, {"test": True}):
                assert expected == host.only("127.0.1.1", ["foo.bar", "foo"])


def test_only_fail():
    """
    Test only() when state change fails
    """
    expected = {
        "name": "127.0.1.1",
        "changes": {},
        "result": False,
        "comment": "hosts.set_host failed to change 127.0.1.1"
        + ' from "foo.bar" to "foo.bar foo"',
    }
    mock = MagicMock(return_value=["foo.bar"])
    with patch.dict(host.__salt__, {"hosts.get_alias": mock}):
        mock = MagicMock(return_value=False)
        with patch.dict(host.__salt__, {"hosts.set_host": mock}):
            with patch.dict(host.__opts__, {"test": False}):
                assert expected == host.only("127.0.1.1", ["foo.bar", "foo"])


def test_only_success():
    """
    Test only() when state successfully changes
    """
    expected = {
        "name": "127.0.1.1",
        "changes": {"127.0.1.1": {"old": "foo.bar", "new": "foo.bar foo"}},
        "result": True,
        "comment": "successfully changed 127.0.1.1"
        + ' from "foo.bar" to "foo.bar foo"',
    }
    mock = MagicMock(return_value=["foo.bar"])
    with patch.dict(host.__salt__, {"hosts.get_alias": mock}):
        mock = MagicMock(return_value=True)
        with patch.dict(host.__salt__, {"hosts.set_host": mock}):
            with patch.dict(host.__opts__, {"test": False}):
                assert expected == host.only("127.0.1.1", ["foo.bar", "foo"])



"""
    :codeauthor: Rahul Handay <rahulha@saltstack.com>
"""

import pytest

import salt.states.mdadm_raid as mdadm
from tests.support.mock import MagicMock, patch


@pytest.fixture
def configure_loader_modules():
    return {mdadm: {}}


def test_present():
    """
    Test to verify that the raid is present
    """
    ret = [
        {
            "changes": {},
            "comment": "Raid salt already present.",
            "name": "salt",
            "result": True,
        },
        {
            "changes": {},
            "comment": (
                "Devices are a mix of RAID constituents with multiple MD_UUIDs:"
                " ['6be5fc45:05802bba:1c2d6722:666f0e03',"
                " 'ffffffff:ffffffff:ffffffff:ffffffff']."
            ),
            "name": "salt",
            "result": False,
        },
        {
            "changes": {},
            "comment": "Raid will be created with: True",
            "name": "salt",
            "result": None,
        },
        {
            "changes": {},
            "comment": "Raid salt failed to be created.",
            "name": "salt",
            "result": False,
        },
        {
            "changes": {"uuid": "6be5fc45:05802bba:1c2d6722:666f0e03"},
            "comment": "Raid salt created.",
            "name": "salt",
            "result": True,
        },
        {
            "changes": {
                "added": ["dev1"],
                "uuid": "6be5fc45:05802bba:1c2d6722:666f0e03",
            },
            "comment": "Raid salt assembled. Added new device dev1 to salt.\n",
            "name": "salt",
            "result": True,
        },
        {
            "changes": {"added": ["dev1"]},
            "comment": "Raid salt already present. Added new device dev1 to salt.\n",
            "name": "salt",
            "result": True,
        },
        {
            "changes": {},
            "comment": "Raid salt failed to be assembled.",
            "name": "salt",
            "result": False,
        },
    ]

    mock_raid_list_exists = MagicMock(
        return_value={"salt": {"uuid": "6be5fc45:05802bba:1c2d6722:666f0e03"}}
    )
    mock_raid_list_missing = MagicMock(return_value={})

    mock_file_access_ok = MagicMock(return_value=True)

    mock_raid_examine_ok = MagicMock(
        return_value={"MD_UUID": "6be5fc45:05802bba:1c2d6722:666f0e03"}
    )
    mock_raid_examine_missing = MagicMock(return_value={})

    mock_raid_create_success = MagicMock(return_value=True)
    mock_raid_create_fail = MagicMock(return_value=False)

    mock_raid_assemble_success = MagicMock(return_value=True)
    mock_raid_assemble_fail = MagicMock(return_value=False)

    mock_raid_add_success = MagicMock(return_value=True)

    mock_raid_save_config = MagicMock(return_value=True)

    with patch.dict(
        mdadm.__salt__,
        {
            "raid.list": mock_raid_list_exists,
            "file.access": mock_file_access_ok,
            "raid.examine": mock_raid_examine_ok,
        },
    ):
        with patch.dict(mdadm.__opts__, {"test": False}):
            assert mdadm.present("salt", 5, "dev0") == ret[0]

    mock_raid_examine_mixed = MagicMock(
        side_effect=[
            {"MD_UUID": "6be5fc45:05802bba:1c2d6722:666f0e03"},
            {"MD_UUID": "ffffffff:ffffffff:ffffffff:ffffffff"},
        ]
    )
    with patch.dict(
        mdadm.__salt__,
        {
            "raid.list": mock_raid_list_missing,
            "file.access": mock_file_access_ok,
            "raid.examine": mock_raid_examine_mixed,
        },
    ):
        with patch.dict(mdadm.__opts__, {"test": False}):
            assert mdadm.present("salt", 5, ["dev0", "dev1"]) == ret[1]

    with patch.dict(
        mdadm.__salt__,
        {
            "raid.list": mock_raid_list_missing,
            "file.access": mock_file_access_ok,
            "raid.examine": mock_raid_examine_missing,
            "raid.create": mock_raid_create_success,
        },
    ):
        with patch.dict(mdadm.__opts__, {"test": True}):
            assert mdadm.present("salt", 5, "dev0") == ret[2]

    with patch.dict(
        mdadm.__salt__,
        {
            "raid.list": mock_raid_list_missing,
            "file.access": mock_file_access_ok,
            "raid.examine": mock_raid_examine_missing,
            "raid.create": mock_raid_create_fail,
        },
    ):
        with patch.dict(mdadm.__opts__, {"test": False}):
            assert mdadm.present("salt", 5, "dev0") == ret[3]

    mock_raid_list_create = MagicMock(
        side_effect=[{}, {"salt": {"uuid": "6be5fc45:05802bba:1c2d6722:666f0e03"}}]
    )
    with patch.dict(
        mdadm.__salt__,
        {
            "raid.list": mock_raid_list_create,
            "file.access": mock_file_access_ok,
            "raid.examine": mock_raid_examine_missing,
            "raid.create": mock_raid_create_success,
            "raid.save_config": mock_raid_save_config,
        },
    ):
        with patch.dict(mdadm.__opts__, {"test": False}):
            assert mdadm.present("salt", 5, "dev0") == ret[4]

    mock_raid_examine_replaced = MagicMock(
        side_effect=[{"MD_UUID": "6be5fc45:05802bba:1c2d6722:666f0e03"}, {}]
    )
    mock_raid_list_create = MagicMock(
        side_effect=[{}, {"salt": {"uuid": "6be5fc45:05802bba:1c2d6722:666f0e03"}}]
    )
    with patch.dict(
        mdadm.__salt__,
        {
            "raid.list": mock_raid_list_create,
            "file.access": mock_file_access_ok,
            "raid.examine": mock_raid_examine_replaced,
            "raid.assemble": mock_raid_assemble_success,
            "raid.add": mock_raid_add_success,
            "raid.save_config": mock_raid_save_config,
        },
    ):
        with patch.dict(mdadm.__opts__, {"test": False}):
            assert mdadm.present("salt", 5, ["dev0", "dev1"]) == ret[5]

    mock_raid_examine_replaced = MagicMock(
        side_effect=[{"MD_UUID": "6be5fc45:05802bba:1c2d6722:666f0e03"}, {}]
    )
    with patch.dict(
        mdadm.__salt__,
        {
            "raid.list": mock_raid_list_exists,
            "file.access": mock_file_access_ok,
            "raid.examine": mock_raid_examine_replaced,
            "raid.add": mock_raid_add_success,
            "raid.save_config": mock_raid_save_config,
        },
    ):
        with patch.dict(mdadm.__opts__, {"test": False}):
            assert mdadm.present("salt", 5, ["dev0", "dev1"]) == ret[6]

    mock_raid_examine_replaced = MagicMock(
        side_effect=[{"MD_UUID": "6be5fc45:05802bba:1c2d6722:666f0e03"}, {}]
    )
    with patch.dict(
        mdadm.__salt__,
        {
            "raid.list": mock_raid_list_missing,
            "file.access": mock_file_access_ok,
            "raid.examine": mock_raid_examine_replaced,
            "raid.assemble": mock_raid_assemble_fail,
        },
    ):
        with patch.dict(mdadm.__opts__, {"test": False}):
            assert mdadm.present("salt", 5, ["dev0", "dev1"]) == ret[7]


def test_absent():
    """
    Test to verify that the raid is absent
    """
    ret = [
        {
            "changes": {},
            "comment": "Raid salt already absent",
            "name": "salt",
            "result": True,
        },
        {
            "changes": {},
            "comment": "Raid saltstack is set to be destroyed",
            "name": "saltstack",
            "result": None,
        },
        {
            "changes": {},
            "comment": "Raid saltstack has been destroyed",
            "name": "saltstack",
            "result": True,
        },
    ]

    mock = MagicMock(return_value=["saltstack"])
    with patch.dict(mdadm.__salt__, {"raid.list": mock}):
        assert mdadm.absent("salt") == ret[0]

        with patch.dict(mdadm.__opts__, {"test": True}):
            assert mdadm.absent("saltstack") == ret[1]

        with patch.dict(mdadm.__opts__, {"test": False}):
            mock = MagicMock(return_value=True)
            with patch.dict(mdadm.__salt__, {"raid.destroy": mock}):
                assert mdadm.absent("saltstack") == ret[2]



"""
Manage LXD images.

.. versionadded:: 2019.2.0

.. link: https://github.com/lxc/pylxd/blob/master/doc/source/installation.rst

.. note:

    - :role:`pylxd <link>` version 2 is required to let this work,
      currently only available via pip.

        To install on Ubuntu:

        $ apt-get install libssl-dev python-pip
        $ pip install -U pylxd

    - you need lxd installed on the minion
      for the init() and version() methods.

    - for the config_get() and config_get() methods
      you need to have lxd-client installed.


:maintainer: René Jochum <rene@jochums.at>
:maturity: new
:depends: python-pylxd
:platform: Linux
"""


from salt.exceptions import CommandExecutionError, SaltInvocationError

__docformat__ = "restructuredtext en"

__virtualname__ = "lxd_image"


def __virtual__():
    """
    Only load if the lxd module is available in __salt__
    """
    if "lxd.version" in __salt__:
        return __virtualname__
    return (False, "lxd module could not be loaded")


def present(
    name,
    source,
    aliases=None,
    public=None,
    auto_update=None,
    remote_addr=None,
    cert=None,
    key=None,
    verify_cert=True,
):
    """
    Ensure an image exists, copy it else from source

    name :
        An alias of the image, this is used to check if the image exists and
        it will be added as alias to the image on copy/create.

    source :
        Source dict.

        For an LXD to LXD copy:

        .. code-block:: yaml

            source:
                type: lxd
                name: ubuntu/xenial/amd64  # This can also be a fingerprint.
                remote_addr: https://images.linuxcontainers.org:8443
                cert: ~/.config/lxd/client.crt
                key: ~/.config/lxd/client.key
                verify_cert: False

        .. attention:

            For this kind of remote you also need to provide:
            - a https:// remote_addr
            - a cert and key
            - verify_cert

        From file:

        .. code-block:: yaml

            source:
                type: file
                filename: salt://lxd/files/busybox.tar.xz
                saltenv: base

        From simplestreams:

        .. code-block:: yaml

            source:
                type: simplestreams
                server: https://cloud-images.ubuntu.com/releases
                name: xenial/amd64

        From an URL:

        .. code-block:: yaml

            source:
                type: url
                url: https://dl.stgraber.org/lxd

    aliases :
        List of aliases to append, can be empty.

    public :
        Make this image public available on this instance?
            None on source_type LXD means copy source
            None on source_type file means False

    auto_update :
        Try to auto-update from the original source?
            None on source_type LXD means copy source
            source_type file does not have auto-update.

    remote_addr :
        An URL to a remote Server, you also have to give cert and key if you
        provide remote_addr!

        Examples:
            https://myserver.lan:8443
            /var/lib/mysocket.sock

    cert :
        PEM Formatted SSL Zertifikate.

        Examples:
            ~/.config/lxc/client.crt

    key :
        PEM Formatted SSL Key.

        Examples:
            ~/.config/lxc/client.key

    verify_cert : True
        Wherever to verify the cert, this is by default True
        but in the most cases you want to set it off as LXD
        normally uses self-signed certificates.
    """
    if aliases is None:
        aliases = []

    # Create a copy of aliases, since we're modifying it here
    aliases = aliases[:]
    ret = {
        "name": name,
        "source": source,
        "aliases": aliases,
        "public": public,
        "auto_update": auto_update,
        "remote_addr": remote_addr,
        "cert": cert,
        "key": key,
        "verify_cert": verify_cert,
        "changes": {},
    }

    image = None
    try:
        image = __salt__["lxd.image_get_by_alias"](
            name, remote_addr, cert, key, verify_cert, _raw=True
        )
    except CommandExecutionError as e:
        return _error(ret, str(e))
    except SaltInvocationError as e:
        # Image not found
        pass

    if image is None:
        if __opts__["test"]:
            # Test is on, just return that we would create the image
            msg = 'Would create the image "{}"'.format(name)
            ret["changes"] = {"created": msg}
            return _unchanged(ret, msg)

        try:
            if source["type"] == "lxd":
                image = __salt__["lxd.image_copy_lxd"](
                    source["name"],
                    src_remote_addr=source["remote_addr"],
                    src_cert=source["cert"],
                    src_key=source["key"],
                    src_verify_cert=source.get("verify_cert", True),
                    remote_addr=remote_addr,
                    cert=cert,
                    key=key,
                    verify_cert=verify_cert,
                    aliases=aliases,
                    public=public,
                    auto_update=auto_update,
                    _raw=True,
                )

            if source["type"] == "file":
                if "saltenv" not in source:
                    source["saltenv"] = __env__
                image = __salt__["lxd.image_from_file"](
                    source["filename"],
                    remote_addr=remote_addr,
                    cert=cert,
                    key=key,
                    verify_cert=verify_cert,
                    aliases=aliases,
                    public=False if public is None else public,
                    saltenv=source["saltenv"],
                    _raw=True,
                )

            if source["type"] == "simplestreams":
                image = __salt__["lxd.image_from_simplestreams"](
                    source["server"],
                    source["name"],
                    remote_addr=remote_addr,
                    cert=cert,
                    key=key,
                    verify_cert=verify_cert,
                    aliases=aliases,
                    public=False if public is None else public,
                    auto_update=False if auto_update is None else auto_update,
                    _raw=True,
                )

            if source["type"] == "url":
                image = __salt__["lxd.image_from_url"](
                    source["url"],
                    remote_addr=remote_addr,
                    cert=cert,
                    key=key,
                    verify_cert=verify_cert,
                    aliases=aliases,
                    public=False if public is None else public,
                    auto_update=False if auto_update is None else auto_update,
                    _raw=True,
                )
        except CommandExecutionError as e:
            return _error(ret, str(e))

    # Sync aliases
    if name not in aliases:
        aliases.append(name)

    old_aliases = {str(a["name"]) for a in image.aliases}
    new_aliases = set(map(str, aliases))

    alias_changes = []
    # Removed aliases
    for k in old_aliases.difference(new_aliases):
        if not __opts__["test"]:
            __salt__["lxd.image_alias_delete"](image, k)
            alias_changes.append('Removed alias "{}"'.format(k))
        else:
            alias_changes.append('Would remove alias "{}"'.format(k))

    # New aliases
    for k in new_aliases.difference(old_aliases):
        if not __opts__["test"]:
            __salt__["lxd.image_alias_add"](image, k, "")
            alias_changes.append('Added alias "{}"'.format(k))
        else:
            alias_changes.append('Would add alias "{}"'.format(k))

    if alias_changes:
        ret["changes"]["aliases"] = alias_changes

    # Set public
    if public is not None and image.public != public:
        if not __opts__["test"]:
            ret["changes"]["public"] = "Setting the image public to {!s}".format(public)
            image.public = public
            __salt__["lxd.pylxd_save_object"](image)
        else:
            ret["changes"]["public"] = "Would set public to {!s}".format(public)

    if __opts__["test"] and ret["changes"]:
        return _unchanged(ret, "Would do {} changes".format(len(ret["changes"].keys())))

    return _success(ret, "{} changes".format(len(ret["changes"].keys())))


def absent(name, remote_addr=None, cert=None, key=None, verify_cert=True):
    """
    name :
        An alias or fingerprint of the image to check and delete.

    remote_addr :
        An URL to a remote Server, you also have to give cert and key if you
        provide remote_addr!

        Examples:
            https://myserver.lan:8443
            /var/lib/mysocket.sock

    cert :
        PEM Formatted SSL Zertifikate.

        Examples:
            ~/.config/lxc/client.crt

    key :
        PEM Formatted SSL Key.

        Examples:
            ~/.config/lxc/client.key

    verify_cert : True
        Wherever to verify the cert, this is by default True
        but in the most cases you want to set it off as LXD
        normally uses self-signed certificates.
    """
    ret = {
        "name": name,
        "remote_addr": remote_addr,
        "cert": cert,
        "key": key,
        "verify_cert": verify_cert,
        "changes": {},
    }
    image = None
    try:
        image = __salt__["lxd.image_get_by_alias"](
            name, remote_addr, cert, key, verify_cert, _raw=True
        )
    except CommandExecutionError as e:
        return _error(ret, str(e))
    except SaltInvocationError as e:
        try:
            image = __salt__["lxd.image_get"](
                name, remote_addr, cert, key, verify_cert, _raw=True
            )
        except CommandExecutionError as e:
            return _error(ret, str(e))
        except SaltInvocationError as e:
            return _success(ret, 'Image "{}" not found.'.format(name))

    if __opts__["test"]:
        ret["changes"] = {"removed": 'Image "{}" would get deleted.'.format(name)}
        return _success(ret, ret["changes"]["removed"])

    __salt__["lxd.image_delete"](image)

    ret["changes"] = {"removed": 'Image "{}" has been deleted.'.format(name)}
    return _success(ret, ret["changes"]["removed"])


def _success(ret, success_msg):
    ret["result"] = True
    ret["comment"] = success_msg
    if "changes" not in ret:
        ret["changes"] = {}
    return ret


def _unchanged(ret, msg):
    ret["result"] = None
    ret["comment"] = msg
    if "changes" not in ret:
        ret["changes"] = {}
    return ret


def _error(ret, err_msg):
    ret["result"] = False
    ret["comment"] = err_msg
    if "changes" not in ret:
        ret["changes"] = {}
    return ret



"""
    :codeauthor: Erik Johnson <erik@saltstack.com>
"""


import copy
import logging
import os
import subprocess

import pytest

import salt.modules.git as git_mod  # Don't potentially shadow GitPython
from salt.utils.versions import Version
from tests.support.mock import MagicMock, Mock, patch

log = logging.getLogger(__name__)


@pytest.fixture(scope="module")
def worktree_root():
    return "/tmp/salt-tests-tmpdir/main"


@pytest.fixture(scope="module")
def worktree_info(worktree_root):
    worktree_info = {
        worktree_root: {
            "HEAD": "119f025073875a938f2456f5ffd7d04e79e5a427",
            "branch": "refs/heads/master",
            "stale": False,
        },
        "/tmp/salt-tests-tmpdir/worktree1": {
            "HEAD": "d8d19cf75d7cc3bdc598dc2d472881d26b51a6bf",
            "branch": "refs/heads/worktree1",
            "stale": False,
        },
        "/tmp/salt-tests-tmpdir/worktree2": {
            "HEAD": "56332ca504aa8b37bb62b54272d52b1d6d832629",
            "branch": "refs/heads/worktree2",
            "stale": True,
        },
        "/tmp/salt-tests-tmpdir/worktree3": {
            "HEAD": "e148ea2d521313579f661373fbb93a48a5a6d40d",
            "branch": "detached",
            "tags": ["v1.1"],
            "stale": False,
        },
        "/tmp/salt-tests-tmpdir/worktree4": {
            "HEAD": "6bbac64d3ad5582b3147088a708952df185db020",
            "branch": "detached",
            "stale": True,
        },
    }

    return worktree_info


def _git_version():
    git_version = subprocess.Popen(
        ["git", "--version"],
        shell=False,
        close_fds=True,
        stdout=subprocess.PIPE,
        stderr=subprocess.PIPE,
    ).communicate()[0]
    if not git_version:
        log.error("Git not installed")
        return False
    log.debug("Detected git version %s", git_version)
    return Version(git_version.split()[-1])


@pytest.fixture
def configure_loader_modules():
    return {git_mod: {"__utils__": {"ssh.key_is_encrypted": Mock(return_value=False)}}}


def test_list_worktrees(worktree_info, worktree_root):
    """
    This tests git.list_worktrees
    """

    def _build_worktree_output(path):
        """
        Build 'git worktree list' output for a given path
        """
        return "worktree {}\nHEAD {}\n{}\n".format(
            path,
            worktree_info[path]["HEAD"],
            "branch {}".format(worktree_info[path]["branch"])
            if worktree_info[path]["branch"] != "detached"
            else "detached",
        )

    # Build dict for _cmd_run_side_effect below. Start with the output from
    # 'git worktree list'.
    _cmd_run_values = {
        "git worktree list --porcelain": "\n".join(
            [_build_worktree_output(x) for x in worktree_info]
        ),
        "git --version": "git version 2.7.0",
    }
    # Add 'git tag --points-at' output for detached HEAD worktrees with
    # tags pointing at HEAD.
    for path in worktree_info:
        if worktree_info[path]["branch"] != "detached":
            continue
        key = "git tag --points-at " + worktree_info[path]["HEAD"]
        _cmd_run_values[key] = "\n".join(worktree_info[path].get("tags", []))

    def _cmd_run_side_effect(key, **kwargs):
        # Not using dict.get() here because we want to know if
        # _cmd_run_values doesn't account for all uses of cmd.run_all.
        return {
            "stdout": _cmd_run_values[" ".join(key)],
            "stderr": "",
            "retcode": 0,
            "pid": 12345,
        }

    def _isdir_side_effect(key):
        # os.path.isdir() would return True on a non-stale worktree
        return not worktree_info[key].get("stale", False)

    # Build return dict for comparison
    worktree_ret = copy.deepcopy(worktree_info)
    for key in worktree_ret:
        ptr = worktree_ret.get(key)
        ptr["detached"] = ptr["branch"] == "detached"
        ptr["branch"] = (
            None if ptr["detached"] else ptr["branch"].replace("refs/heads/", "", 1)
        )

    cmd_run_mock = MagicMock(side_effect=_cmd_run_side_effect)
    isdir_mock = MagicMock(side_effect=_isdir_side_effect)
    with patch.dict(git_mod.__salt__, {"cmd.run_all": cmd_run_mock}):
        with patch.object(os.path, "isdir", isdir_mock):
            # Test all=True. Include all return data.
            assert (
                git_mod.list_worktrees(worktree_root, all=True, stale=False)
                == worktree_ret
            )
            # Test all=False and stale=False. Exclude stale worktrees from
            # return data.
            assert git_mod.list_worktrees(worktree_root, all=False, stale=False) == {
                x: worktree_ret[x]
                for x in worktree_info
                if not worktree_info[x].get("stale", False)
            }
            # Test stale=True. Exclude non-stale worktrees from return
            # data.
            assert git_mod.list_worktrees(worktree_root, all=False, stale=True) == {
                x: worktree_ret[x]
                for x in worktree_info
                if worktree_info[x].get("stale", False)
            }


def test__git_run_tmp_wrapper():
    """
    When an identity file is specified, make sure we don't attempt to
    remove a temp wrapper that wasn't created. Windows doesn't use temp
    wrappers, and *NIX won't unless no username was specified and the path
    is not executable.
    """
    file_remove_mock = Mock()
    mock_true = Mock(return_value=True)
    mock_false = Mock(return_value=False)
    cmd_mock = MagicMock(return_value={"retcode": 0, "stdout": "", "stderr": ""})
    with patch.dict(
        git_mod.__salt__,
        {
            "file.file_exists": mock_true,
            "file.remove": file_remove_mock,
            "cmd.run_all": cmd_mock,
            "ssh.key_is_encrypted": mock_false,
        },
    ):

        # Non-windows
        with patch("salt.utils.platform.is_windows", mock_false), patch.object(
            git_mod, "_path_is_executable_others", mock_true
        ):

            # Command doesn't really matter here since we're mocking
            git_mod._git_run(
                ["git", "rev-parse", "HEAD"],
                cwd="/some/path",
                user=None,
                identity="/root/.ssh/id_rsa",
            )

            file_remove_mock.assert_not_called()

        file_remove_mock.reset_mock()
        with patch("salt.utils.platform.is_windows", mock_true), patch.object(
            git_mod, "_find_ssh_exe", MagicMock(return_value=r"C:\Git\ssh.exe")
        ):
            # Command doesn't really matter here since we're mocking
            git_mod._git_run(
                ["git", "rev-parse", "HEAD"],
                cwd=r"C:\some\path",
                user=None,
                identity=r"C:\ssh\id_rsa",
            )

            file_remove_mock.assert_not_called()



"""
unit tests for the sqs_events engine
"""
import pytest

import salt.engines.sqs_events as sqs_events
from tests.support.mock import MagicMock, patch

pytestmark = [
    pytest.mark.skipif(
        sqs_events.HAS_BOTO is False, reason="The boto library is not installed"
    )
]


@pytest.fixture
def configure_loader_modules():
    return {sqs_events: {}}


@pytest.fixture
def mock_sqs():
    with patch("salt.engines.sqs_events.boto.sqs") as mock_sqs:
        yield mock_sqs


def sample_msg():
    fake_msg = MagicMock()
    fake_msg.get_body.return_value = "This is a test message"
    fake_msg.delete.return_value = True
    return fake_msg


# 'present' function tests: 1
def test_no_queue_present(mock_sqs):
    """
    Test to ensure the SQS engine logs a warning when queue not present
    """
    with patch("salt.engines.sqs_events.log") as mock_logging:
        with patch("time.sleep", return_value=None) as mock_sleep:
            q = None
            q_name = "mysqs"
            mock_fire = MagicMock(return_value=True)
            sqs_events._process_queue(q, q_name, mock_fire)
            assert mock_logging.warning.called
            assert not mock_sqs.queue.Queue().get_messages.called


def test_minion_message_fires(mock_sqs):
    """
    Test SQS engine correctly gets and fires messages on minion
    """
    msgs = [sample_msg(), sample_msg()]
    mock_sqs.queue.Queue().get_messages.return_value = msgs
    q = mock_sqs.queue.Queue()
    q_name = "mysqs"
    mock_event = MagicMock(return_value=True)
    mock_fire = MagicMock(return_value=True)
    with patch.dict(sqs_events.__salt__, {"event.send": mock_event}):
        sqs_events._process_queue(q, q_name, mock_fire)
        assert mock_sqs.queue.Queue().get_messages.called
        assert all(x.delete.called for x in msgs)


def test_master_message_fires(mock_sqs):
    """
    Test SQS engine correctly gets and fires messages on master
    """
    msgs = [sample_msg(), sample_msg()]
    mock_sqs.queue.Queue().get_messages.return_value = msgs
    q = mock_sqs.queue.Queue()
    q_name = "mysqs"
    mock_fire = MagicMock(return_value=True)
    sqs_events._process_queue(q, q_name, mock_fire)
    assert mock_sqs.queue.Queue().get_messages.called, len(msgs)
    assert mock_fire.called, len(msgs)



"""
    tests.pytests.unit.beacons.test_diskusage
    ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

    Disk usage beacon test cases
"""
from collections import namedtuple

import pytest

import salt.beacons.diskusage as diskusage
from tests.support.mock import MagicMock, Mock, patch


@pytest.fixture
def configure_loader_modules():
    return {}


@pytest.fixture
def stub_disk_partition():
    return [
        namedtuple("partition", "device mountpoint fstype, opts")(
            "tmpfs", "/mnt/tmp", "tmpfs", "rw,nosuid,nodev,relatime,size=10240k"
        ),
        namedtuple("partition", "device mountpoint fstype, opts")(
            "/dev/disk0s2", "/", "hfs", "rw,local,rootfs,dovolfs,journaled,multilabel"
        ),
    ]


@pytest.fixture
def windows_stub_disk_partition():
    return [
        namedtuple("partition", "device mountpoint fstype, opts")(
            "C:\\", "C:\\", "NTFS", "rw,fixed"
        ),
        namedtuple("partition", "device mountpoint fstype, opts")(
            "D:\\", "D:\\", "CDFS", "ro,cdrom"
        ),
    ]


@pytest.fixture
def stub_disk_usage():
    return [
        namedtuple("usage", "total used free percent")(1000, 500, 500, 50),
        namedtuple("usage", "total used free percent")(100, 75, 25, 25),
    ]


@pytest.fixture
def windows_stub_disk_usage():
    return namedtuple("usage", "total used free percent")(1000, 500, 500, 50)


def test_non_list_config():
    config = {}

    ret = diskusage.validate(config)
    assert ret == (False, "Configuration for diskusage beacon must be a list.")


def test_empty_config():
    config = [{}]

    ret = diskusage.validate(config)
    assert ret == (True, "Valid beacon configuration")


def test_diskusage_match(stub_disk_usage, stub_disk_partition):
    disk_usage_mock = Mock(side_effect=stub_disk_usage)
    with patch("salt.utils.platform.is_windows", MagicMock(return_value=False)), patch(
        "psutil.disk_partitions", MagicMock(return_value=stub_disk_partition)
    ), patch("psutil.disk_usage", disk_usage_mock):
        config = [{"/": "50%"}]

        ret = diskusage.validate(config)
        assert ret == (True, "Valid beacon configuration")

        ret = diskusage.beacon(config)
        assert ret == [{"diskusage": 50, "mount": "/"}]


def test_diskusage_match_no_percent(stub_disk_usage, stub_disk_partition):
    disk_usage_mock = Mock(side_effect=stub_disk_usage)
    with patch("salt.utils.platform.is_windows", MagicMock(return_value=False)), patch(
        "psutil.disk_partitions", MagicMock(return_value=stub_disk_partition)
    ), patch("psutil.disk_usage", disk_usage_mock):

        # Test without the percent
        config = [{"/": 50}]

        ret = diskusage.validate(config)
        assert ret == (True, "Valid beacon configuration")

        ret = diskusage.beacon(config)
        assert ret == [{"diskusage": 50, "mount": "/"}]


def test_diskusage_nomatch(stub_disk_usage, stub_disk_partition):
    disk_usage_mock = Mock(side_effect=stub_disk_usage)
    with patch("salt.utils.platform.is_windows", MagicMock(return_value=False)), patch(
        "psutil.disk_partitions", MagicMock(return_value=stub_disk_partition)
    ), patch("psutil.disk_usage", disk_usage_mock):
        config = [{"/": "70%"}]

        ret = diskusage.validate(config)
        assert ret == (True, "Valid beacon configuration")

        ret = diskusage.beacon(config)
        assert ret != [{"diskusage": 50, "mount": "/"}]


def test_diskusage_match_regex(stub_disk_usage, stub_disk_partition):
    disk_usage_mock = Mock(side_effect=stub_disk_usage)
    with patch("salt.utils.platform.is_windows", MagicMock(return_value=False)), patch(
        "psutil.disk_partitions", MagicMock(return_value=stub_disk_partition)
    ), patch("psutil.disk_usage", disk_usage_mock):
        config = [{"/": "50%"}]

        ret = diskusage.validate(config)
        assert ret == (True, "Valid beacon configuration")

        ret = diskusage.beacon(config)
        assert ret == [{"diskusage": 50, "mount": "/"}]


def test_diskusage_windows_single_slash(
    windows_stub_disk_usage, windows_stub_disk_partition
):
    r"""
    This tests new behavior (C:\)
    """
    disk_usage_mock = Mock(return_value=windows_stub_disk_usage)
    with patch("salt.utils.platform.is_windows", MagicMock(return_value=True)):
        with patch(
            "psutil.disk_partitions",
            MagicMock(return_value=windows_stub_disk_partition),
        ), patch("psutil.disk_usage", disk_usage_mock):
            config = [{"C:\\": "50%"}]

            ret = diskusage.validate(config)
            assert ret == (True, "Valid beacon configuration")

            ret = diskusage.beacon(config)
            assert ret == [{"diskusage": 50, "mount": "C:\\"}]


def test_diskusage_windows_double_slash(
    windows_stub_disk_usage, windows_stub_disk_partition
):
    """
    This tests original behavior (C:\\)
    """
    disk_usage_mock = Mock(return_value=windows_stub_disk_usage)
    with patch("salt.utils.platform.is_windows", MagicMock(return_value=True)):
        with patch(
            "psutil.disk_partitions",
            MagicMock(return_value=windows_stub_disk_partition),
        ), patch("psutil.disk_usage", disk_usage_mock):
            config = [{"C:\\\\": "50%"}]

            ret = diskusage.validate(config)
            assert ret == (True, "Valid beacon configuration")

            ret = diskusage.beacon(config)
            assert ret == [{"diskusage": 50, "mount": "C:\\"}]


def test_diskusage_windows_lowercase(
    windows_stub_disk_usage, windows_stub_disk_partition
):
    r"""
    This tests lowercase drive letter (c:\)
    """
    disk_usage_mock = Mock(return_value=windows_stub_disk_usage)
    with patch("salt.utils.platform.is_windows", MagicMock(return_value=True)):
        with patch(
            "psutil.disk_partitions",
            MagicMock(return_value=windows_stub_disk_partition),
        ), patch("psutil.disk_usage", disk_usage_mock):
            config = [{"c:\\": "50%"}]

            ret = diskusage.validate(config)
            assert ret == (True, "Valid beacon configuration")

            ret = diskusage.beacon(config)
            assert ret == [{"diskusage": 50, "mount": "C:\\"}]


def test_diskusage_windows_match_regex(
    windows_stub_disk_usage, windows_stub_disk_partition
):
    disk_usage_mock = Mock(return_value=windows_stub_disk_usage)
    with patch("salt.utils.platform.is_windows", MagicMock(return_value=True)):
        with patch(
            "psutil.disk_partitions",
            MagicMock(return_value=windows_stub_disk_partition),
        ), patch("psutil.disk_usage", disk_usage_mock):
            config = [{"^[a-zA-Z]:\\": "50%"}]

            ret = diskusage.validate(config)
            assert ret == (True, "Valid beacon configuration")

            ret = diskusage.beacon(config)
            _expected = [
                {"diskusage": 50, "mount": "C:\\"},
                {"diskusage": 50, "mount": "D:\\"},
            ]
            assert ret == _expected



import pytest

import salt.modules.config as configmod
import salt.modules.postgres as postgres
from tests.support.mock import MagicMock, patch

# 'md5' + md5('password' + 'username')
md5_pw = "md55a231fcdb710d73268c4f44283487ba2"

scram_pw = (
    "SCRAM-SHA-256$4096:wLr5nqC+3F+r7FdQPnB+nA==$"
    "0hn08ZdX8kirGaL4TM0j13digH9Wl365OOzCtAuF2pE=:"
    "LzAh/MGUdjYkdbDzcOKpfGwa3WwPUsyGcY+TEnSpcto="
)

test_privileges_list_function_csv = (
    'name\n"{baruwatest=X/baruwatest,bayestest=r/baruwatest,baruwa=X*/baruwatest}"\n'
)


@pytest.fixture
def configure_loader_modules():
    return {
        postgres: {
            "__grains__": {"os_family": "Linux"},
            "__salt__": {
                "config.option": MagicMock(),
                "cmd.run_all": MagicMock(),
                "file.chown": MagicMock(),
                "file.remove": MagicMock(),
            },
        },
        configmod: {},
    }


def idfn(val):
    if val == md5_pw:
        return "md5_pw"
    if val == scram_pw:
        return "scram_pw"


@pytest.mark.parametrize(
    "role,password,verifier,method,result",
    [
        ("username", "password", md5_pw, "md5", True),
        ("another", "password", md5_pw, "md5", False),
        ("username", "another", md5_pw, "md5", False),
        ("username", md5_pw, md5_pw, "md5", True),
        ("username", "md5another", md5_pw, "md5", False),
        ("username", "password", md5_pw, True, True),
        ("another", "password", md5_pw, True, False),
        ("username", "another", md5_pw, True, False),
        ("username", md5_pw, md5_pw, True, True),
        ("username", "md5another", md5_pw, True, False),
        (None, "password", scram_pw, "scram-sha-256", True),
        (None, "another", scram_pw, "scram-sha-256", False),
        (None, scram_pw, scram_pw, "scram-sha-256", True),
        (None, "SCRAM-SHA-256$4096:AAAA$AAAA:AAAA", scram_pw, "scram-sha-256", False),
        (None, "SCRAM-SHA-256$foo", scram_pw, "scram-sha-256", False),
        (None, "password", "password", False, True),
        (None, "another", "password", False, False),
        (None, "password", "password", "foo", False),
        ("username", "password", md5_pw, "scram-sha-256", False),
        ("username", "password", scram_pw, "md5", False),
        # Code does not currently check role of pre-hashed md5 passwords
        pytest.param("another", md5_pw, md5_pw, "md5", False, marks=pytest.mark.xfail),
    ],
    ids=idfn,
)
def test_verify_password(role, password, verifier, method, result):
    assert postgres._verify_password(role, password, verifier, method) == result


def test_has_privileges_with_function():
    with patch(
        "salt.modules.postgres._run_psql",
        MagicMock(
            return_value={"retcode": 0, "stdout": test_privileges_list_function_csv}
        ),
    ), patch("salt.utils.path.which", MagicMock(return_value="/usr/bin/pgsql")):
        ret = postgres.has_privileges(
            "baruwa",
            "awl",
            "function",
            "EXECUTE",
            grant_option=True,
            maintenance_db="db_name",
            runas="user",
            host="testhost",
            port="testport",
            user="testuser",
            password="testpassword",
        )

        assert ret is True

        query = (
            "COPY (SELECT rolname AS name "
            "FROM pg_catalog.pg_proc p "
            "JOIN pg_catalog.pg_namespace n "
            "ON n.oid = p.pronamespace "
            "JOIN pg_catalog.pg_roles r "
            "ON p.proowner = r.oid "
            "WHERE nspname = 'public' "
            "AND p.oid::regprocedure::text = 'awl' "
            "ORDER BY proname, proargtypes) TO STDOUT WITH CSV HEADER"
        )

        postgres._run_psql.assert_any_call(
            [
                "/usr/bin/pgsql",
                "--no-align",
                "--no-readline",
                "--no-psqlrc",
                "--no-password",
                "--username",
                "testuser",
                "--host",
                "testhost",
                "--port",
                "testport",
                "--dbname",
                "db_name",
                "-v",
                "datestyle=ISO,MDY",
                "-c",
                query,
            ],
            host="testhost",
            port="testport",
            password="testpassword",
            user="testuser",
            runas="user",
        )


def test__runpsql_with_timeout():
    cmd_run_mock = MagicMock()
    postgres_opts = {
        "config.option": configmod.option,
        "cmd.run_all": cmd_run_mock,
    }
    kwargs = {
        "reset_system_locale": False,
        "clean_env": True,
        "runas": "saltuser",
        "python_shell": False,
    }
    with patch.dict(postgres.__salt__, postgres_opts):
        with patch.dict(
            configmod.__opts__, {"postgres.timeout": 60, "postgres.pass": None}
        ):
            postgres._run_psql("fakecmd", runas="saltuser")
            cmd_run_mock.assert_called_with("fakecmd", timeout=60, **kwargs)
        with patch.dict(configmod.__opts__, {"postgres.pass": None}):
            postgres._run_psql("fakecmd", runas="saltuser")
            cmd_run_mock.assert_called_with("fakecmd", timeout=0, **kwargs)


def test__run_initdb_with_timeout():
    cmd_run_mock = MagicMock(return_value={})
    postgres_opts = {
        "config.option": configmod.option,
        "cmd.run_all": cmd_run_mock,
    }
    kwargs = {
        "clean_env": True,
        "runas": "saltuser",
        "python_shell": False,
    }
    cmd_str = "/fake/path --pgdata=fakename --username=saltuser --auth=password --encoding=UTF8"
    with patch.dict(postgres.__salt__, postgres_opts):
        with patch.object(postgres, "_find_pg_binary", return_value="/fake/path"):
            with patch.dict(
                configmod.__opts__, {"postgres.timeout": 60, "postgres.pass": None}
            ):
                postgres._run_initdb("fakename", runas="saltuser")
                cmd_run_mock.assert_called_with(cmd_str, timeout=60, **kwargs)
            with patch.dict(configmod.__opts__, {"postgres.pass": None}):
                postgres._run_initdb("fakename", runas="saltuser")
                cmd_run_mock.assert_called_with(cmd_str, timeout=0, **kwargs)



"""
    :codeauthor: Mike Wiebe <@mikewiebe>
"""

# Copyright (c) 2019 Cisco and/or its affiliates.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import pytest

import salt.proxy.nxos as nxos_proxy
import salt.utils.nxos as nxos_utils
from tests.support.mock import create_autospec, patch
from tests.unit.modules.nxos.nxos_grains import n9k_grains
from tests.unit.modules.nxos.nxos_show_cmd_output import n9k_show_ver_list


@pytest.fixture
def configure_loader_modules():
    with patch("salt.proxy.nxos.CONNECTION", "nxapi"):
        yield {nxos_proxy: {}}


def test_check_virtual():

    """UT: nxos module:check_virtual method - return value"""

    result = nxos_proxy.__virtual__()
    assert "nxos" in result


def test_init():

    """UT: nxos module:init method - nxapi proxy"""

    with patch.object(nxos_proxy, "__opts__", {"proxy": {"connection": "nxapi"}}):
        with patch("salt.proxy.nxos._init_nxapi", autospec=True) as init_nxapi:
            result = nxos_proxy.init()
            assert result == init_nxapi.return_value


def test_init_opts_none():

    """UT: nxos module:init method - __opts__ connection is None"""

    with patch.object(nxos_proxy, "__opts__", {"proxy": {"connection": None}}):
        with patch("salt.proxy.nxos._init_nxapi", autospec=True) as init_nxapi:
            result = nxos_proxy.init()
            assert result == init_nxapi.return_value


def test_init_bad_connection_type():

    """UT: nxos module:init method - bad CONNECTION type"""
    with patch.object(nxos_proxy, "__opts__", {"proxy": {"connection": "unknown"}}):
        assert not nxos_proxy.init()


def test_initialized():

    """UT: nxos module:initialized method - nxapi proxy"""

    with patch(
        "salt.proxy.nxos._initialized_nxapi", autospec=True
    ) as initialized_nxapi:
        result = nxos_proxy.initialized()
        assert result == initialized_nxapi.return_value


def test_ping():

    """UT: nxos module:ping method - nxapi proxy"""

    with patch("salt.proxy.nxos._ping_nxapi", autospec=True) as ping_nxapi:
        result = nxos_proxy.ping()
        assert result == ping_nxapi.return_value


def test_grains():

    """UT: nxos module:grains method - nxapi grains"""

    with patch(
        "salt.proxy.nxos.sendline", autospec=True, return_value=n9k_show_ver_list
    ):
        result = nxos_proxy.grains()
        assert result == n9k_grains


def test_grains_cache_set():

    """UT: nxos module:grains method - nxapi grains cache set"""

    with patch("salt.proxy.nxos.DEVICE_DETAILS", {"grains_cache": n9k_grains["nxos"]}):
        with patch(
            "salt.proxy.nxos.sendline",
            autospec=True,
            return_value=n9k_show_ver_list,
        ):
            result = nxos_proxy.grains()
            assert result == n9k_grains


def test_grains_refresh():

    """UT: nxos module:grains_refresh method - nxapi grains"""

    device_details = {"grains_cache": None}

    with patch("salt.proxy.nxos.DEVICE_DETAILS", device_details):
        with patch("salt.proxy.nxos.grains", autospec=True) as grains:
            result = nxos_proxy.grains_refresh()
            assert nxos_proxy.DEVICE_DETAILS["grains_cache"] == {}
            assert result == grains.return_value


def test_sendline():

    """UT: nxos module:sendline method - nxapi"""

    command = "show version"

    with patch("salt.proxy.nxos._nxapi_request", autospec=True) as nxapi_request:
        result = nxos_proxy.sendline(command)
        assert result == nxapi_request.return_value


def test_proxy_config():

    """UT: nxos module:proxy_config method - nxapi success path"""

    commands = ["feature bgp", "router bgp 65535"]

    with patch("salt.proxy.nxos.DEVICE_DETAILS", {"save_config": False}):
        with patch("salt.proxy.nxos._nxapi_request", autospec=True) as nxapi_request:
            result = nxos_proxy.proxy_config(commands)
            assert result == [commands, nxapi_request.return_value]


def test_proxy_config_save_config():

    """UT: nxos module:proxy_config method - nxapi success path"""

    commands = ["feature bgp", "router bgp 65535"]

    with patch("salt.proxy.nxos.DEVICE_DETAILS", {"save_config": None}):
        with patch("salt.proxy.nxos._nxapi_request", autospec=True) as nxapi_request:
            result = nxos_proxy.proxy_config(commands, save_config=True)
            assert result == [commands, nxapi_request.return_value]


def test__init_nxapi():

    """UT: nxos module:_init_nxapi method - successful connectinon"""

    opts = {"proxy": {"arg1": None}}
    nxapi_request = create_autospec(nxos_utils.nxapi_request, return_value="data")

    with patch("salt.proxy.nxos.DEVICE_DETAILS", {}) as device_details:
        with patch("salt.proxy.nxos.__utils__", {"nxos.nxapi_request": nxapi_request}):
            result = nxos_proxy._init_nxapi(opts)

            assert device_details["initialized"]
            assert device_details["up"]
            assert device_details["save_config"]
            assert result

            nxapi_request.assert_called_with("show clock", **opts["proxy"])


def test_bad__init_nxapi():
    class NXAPIException(Exception):
        pass

    nxapi_request = create_autospec(
        nxos_utils.nxapi_request, side_effect=NXAPIException
    )

    with patch("salt.proxy.nxos.__utils__", {"nxos.nxapi_request": nxapi_request}):
        with patch("salt.proxy.nxos.log", autospec=True) as log:
            with pytest.raises(NXAPIException):
                nxos_proxy._init_nxapi({"proxy": {"host": "HOST"}})
            log.error.assert_called()


def test__initialized_nxapi():

    """UT: nxos module:_initialized_nxapi method"""

    result = nxos_proxy._initialized_nxapi()
    assert not result

    with patch("salt.proxy.nxos.DEVICE_DETAILS", {"initialized": True}):
        result = nxos_proxy._initialized_nxapi()
        assert result


def test__ping_nxapi():

    """UT: nxos module:_ping_nxapi method"""

    result = nxos_proxy._ping_nxapi()
    assert not result

    with patch("salt.proxy.nxos.DEVICE_DETAILS", {"up": True}):
        result = nxos_proxy._ping_nxapi()
        assert result


def test__shutdown_nxapi():

    """UT: nxos module:_shutdown_nxapi method"""

    opts = {"id": "value"}

    with patch("salt.proxy.nxos.log", autospec=True):
        nxos_proxy._shutdown_nxapi()
        # nothing to test


def test__nxapi_request_ssh_return():

    """UT: nxos module:_nxapi_request method - CONNECTION == 'ssh'"""

    commands = "show version"

    with patch("salt.proxy.nxos.CONNECTION", "ssh"):
        result = nxos_proxy._nxapi_request(commands)
        assert "_nxapi_request is not available for ssh proxy" == result


def test__nxapi_request_connect():

    """UT: nxos module:_nxapi_request method"""

    commands = "show version"
    nxapi_request = create_autospec(nxos_utils.nxapi_request, return_value="data")

    with patch("salt.proxy.nxos.DEVICE_DETAILS", {"conn_args": {"arg1": None}}):
        with patch("salt.proxy.nxos.__utils__", {"nxos.nxapi_request": nxapi_request}):
            result = nxos_proxy._nxapi_request(commands)
            assert "data" == result
            nxapi_request.assert_called_with(commands, method="cli_conf", arg1=None)



"""
Test the elasticsearch returner
"""
import pytest

import salt.returners.elasticsearch_return as elasticsearch_return
from tests.support.mock import MagicMock, patch


@pytest.fixture
def configure_loader_modules():
    return {elasticsearch_return: {}}


def test__virtual_no_elasticsearch():
    """
    Test __virtual__ function when elasticsearch is not installed
    and the elasticsearch module is not available
    """
    result = elasticsearch_return.__virtual__()
    expected = (
        False,
        "Elasticsearch module not availble.  Check that the elasticsearch library"
        " is installed.",
    )
    assert expected == result


def test__virtual_with_elasticsearch():
    """
    Test __virtual__ function when elasticsearch
    and the elasticsearch module is not available
    """
    with patch.dict(
        elasticsearch_return.__salt__, {"elasticsearch.index_exists": MagicMock()}
    ):
        result = elasticsearch_return.__virtual__()
        expected = "elasticsearch"
        assert expected == result



"""
Test the win_task execution module
"""
from datetime import datetime

import pytest

import salt.modules.win_task as win_task

pytestmark = [
    pytest.mark.skip_unless_on_windows,
    pytest.mark.destructive_test,
]


@pytest.fixture(scope="function")
def base_task():
    task_name = "SaltTest"
    result = win_task.create_task(
        task_name,
        user_name="System",
        force=True,
        action_type="Execute",
        cmd="c:\\salt\\salt-call.bat",
    )
    assert result is True
    yield task_name
    result = win_task.delete_task(task_name)
    assert result is True


def test_repeat_interval(base_task):
    result = win_task.add_trigger(
        base_task,
        trigger_type="Daily",
        trigger_enabled=True,
        repeat_duration="30 minutes",
        repeat_interval="30 minutes",
    )
    assert result is True

    result = win_task.info(base_task)
    assert result["triggers"][0]["enabled"] is True
    assert result["triggers"][0]["trigger_type"] == "Daily"
    assert result["triggers"][0]["repeat_duration"] == "30 minutes"
    assert result["triggers"][0]["repeat_interval"] == "30 minutes"


def test_repeat_interval_and_indefinitely(base_task):
    result = win_task.add_trigger(
        base_task,
        trigger_type="Daily",
        trigger_enabled=True,
        repeat_duration="Indefinitely",
        repeat_interval="30 minutes",
    )
    assert result is True

    result = win_task.info(base_task)
    assert result["triggers"][0]["enabled"] is True
    assert result["triggers"][0]["trigger_type"] == "Daily"
    assert result["triggers"][0]["repeat_duration"] == "Indefinitely"
    assert result["triggers"][0]["repeat_interval"] == "30 minutes"


def test_edit_task_delete_after(base_task):
    result = win_task.add_trigger(
        base_task,
        trigger_type="Daily",
        trigger_enabled=True,
        end_date=datetime.today().strftime("%Y-%m-%d"),
        end_time="23:59:59",
    )
    assert result is True

    result = win_task.edit_task(base_task, delete_after="30 days")
    assert result is True

    result = win_task.info(base_task)
    assert result["settings"]["delete_after"] == "30 days"

    result = win_task.edit_task(base_task, delete_after=False)
    assert result is True

    result = win_task.info(base_task)
    assert result["settings"]["delete_after"] is False



import datetime
import sys
import warnings

import pytest
from packaging.version import InvalidVersion

import salt.modules.cmdmod
import salt.utils.versions
import salt.version
from salt.utils.versions import LooseVersion, Version
from tests.support.mock import patch


def test_prerelease():
    version = Version("1.2.3a1")
    assert version.release == (1, 2, 3)
    assert version.pre == ("a", 1)
    assert str(version) == "1.2.3a1"

    version = Version("1.2.0")
    assert str(version) == "1.2.0"


@pytest.mark.parametrize(
    "v1,v2,wanted",
    (
        ("1.5.1", "1.5.2b2", -1),
        ("161", "3.10a", 1),
        ("8.02", "8.02", 0),
        ("3.4j", "1996.07.12", InvalidVersion),
        ("3.2.pl0", "3.1.1.6", InvalidVersion),
        ("2g6", "11g", InvalidVersion),
        ("0.9", "2.2", -1),
        ("1.2.1", "1.2", 1),
        ("1.1", "1.2.2", -1),
        ("1.2", "1.1", 1),
        ("1.2.1", "1.2.2", -1),
        ("1.2.2", "1.2", 1),
        ("1.2", "1.2.2", -1),
        ("0.4.0", "0.4", 0),
        ("1.13++", "5.5.kw", InvalidVersion),
        ("1.1.1a1", "1.1.1", -1),
    ),
)
def test_cmp_strict(v1, v2, wanted):
    try:
        v1i = Version(v1)
        v2i = Version(v2)
        if v1i == v2i:
            res = 0
        elif v1i < v2i:
            res = -1
        elif v1i > v2i:
            res = 1
        assert res == wanted, "cmp({}, {}) should be {}, got {}".format(
            v1, v2, wanted, res
        )
    except InvalidVersion:
        if wanted is not InvalidVersion:
            raise AssertionError(
                "cmp({}, {}) shouldn't raise InvalidVersion".format(v1, v2)
            )


@pytest.mark.parametrize(
    "v1,v2,wanted",
    (
        ("1.5.1", "1.5.2b2", -1),
        ("161", "3.10a", 1),
        ("8.02", "8.02", 0),
        ("3.4j", "1996.07.12", -1),
        ("3.2.pl0", "3.1.1.6", 1),
        ("2g6", "11g", -1),
        ("0.960923", "2.2beta29", -1),
        ("1.13++", "5.5.kw", -1),
        # Added by us
        ("3.10.0-514.el7", "3.10.0-514.6.1.el7", 1),
        ("2.2.2", "2.12.1", -1),
        ("2.24.0", "2.23.0.windows.1", 1),
        ("2.23.0.windows.2", "2.23.0.windows.1", 1),
    ),
)
def test_cmp(v1, v2, wanted):
    res = LooseVersion(v1)._cmp(LooseVersion(v2))
    assert res == wanted, "cmp({}, {}) should be {}, got {}".format(v1, v2, wanted, res)


def test_compare():
    ret = salt.utils.versions.compare("1.0", "==", "1.0")
    assert ret

    ret = salt.utils.versions.compare("1.0", "!=", "1.0")
    assert not ret

    with patch.object(salt.utils.versions, "log") as log_mock:
        ret = salt.utils.versions.compare(
            "1.0", "HAH I AM NOT A COMP OPERATOR! I AM YOUR FATHER!", "1.0"
        )
        assert log_mock.error.called


@pytest.mark.parametrize(
    "version",
    (
        "Chlorine",
        3007,
        (3007, 0),
        salt.version.SaltVersionsInfo.CHLORINE,
    ),
)
def test_warn_until_good_version_argument(version):
    with pytest.raises(
        RuntimeError,
        match=(
            r"The warning triggered on filename \'(.*)test_versions.py\', "
            r"line number ([\d]+), is supposed to be shown until version "
            r"3007.0 \(Chlorine\) is released. Current version is now 3009.0 \(Potassium\). "
            r"Please remove the warning."
        ),
    ):
        salt.utils.versions.warn_until(
            version, "Deprecation Message after {version}!", _version_info_=(3009, 0)
        )


def test_warn_until_bad_version_name_raises_runtime_error():
    # Ensure proper behavior
    with warnings.catch_warnings(record=True) as recorded_warnings:
        salt.utils.versions.warn_until(
            "Sodium", "Deprecation Message!", _version_info_=(3000, 0)
        )
        assert "Deprecation Message!" == str(recorded_warnings[0].message)

    with pytest.raises(
        RuntimeError, match="Incorrect spelling for the release name in .*"
    ):
        salt.utils.versions.warn_until(
            "Sudium", "Deprecation Message!", _version_info_=(3000, 0)
        )


def test_kwargs_warn_until():
    # Test invalid version arg
    pytest.raises(RuntimeError, salt.utils.versions.kwargs_warn_until, {}, [])


def test_warn_until_warning_raised(subtests):
    # We *always* want *all* warnings thrown on this module
    warnings.filterwarnings("always", "", DeprecationWarning, __name__)

    def raise_warning(_version_info_=(0, 16, 0)):
        salt.utils.versions.warn_until(
            (0, 17), "Deprecation Message!", _version_info_=_version_info_
        )

    def raise_named_version_warning(_version_info_=(0, 16, 0)):
        salt.utils.versions.warn_until(
            "hydrogen", "Deprecation Message!", _version_info_=_version_info_
        )

    with subtests.test(
        "raise_warning should show warning until version info is >= (0, 17)"
    ):
        with warnings.catch_warnings(record=True) as recorded_warnings:
            raise_warning()
            assert "Deprecation Message!" == str(recorded_warnings[0].message)

    with subtests.test(
        "raise_warning should show warning until version info is >= (0, 17)"
    ):
        with warnings.catch_warnings(record=True) as recorded_warnings:
            raise_named_version_warning()
            assert "Deprecation Message!" == str(recorded_warnings[0].message)

    with subtests.test(
        "the deprecation warning is not issued because we passed _dont_call_warning"
    ):
        with warnings.catch_warnings(record=True) as recorded_warnings:
            salt.utils.versions.warn_until(
                (0, 17), "Foo", _dont_call_warnings=True, _version_info_=(0, 16)
            )
            assert 0 == len(recorded_warnings)

    with subtests.test(
        "Let's set version info to (0, 17), a RuntimeError should be raised"
    ):
        with pytest.raises(
            RuntimeError,
            match=(
                r"The warning triggered on filename \'(.*)test_versions.py\', "
                r"line number ([\d]+), is supposed to be shown until version "
                r"0.17.0 is released. Current version is now 0.17.0. "
                r"Please remove the warning."
            ),
        ):
            raise_warning(_version_info_=(0, 17, 0))

    with subtests.test(
        "Let's set version info to (sys,maxsize, 16, 0), a RuntimeError should be raised"
    ):
        with pytest.raises(
            RuntimeError,
            match=(
                r"The warning triggered on filename \'(.*)test_versions.py\', "
                r"line number ([\d]+), is supposed to be shown until version "
                r"(.*) is released. Current version is now "
                r"([\d.]+). Please remove the warning."
            ),
        ):
            raise_named_version_warning(_version_info_=(sys.maxsize, 16, 0))

    with subtests.test(
        "Even though we're calling warn_until, we pass _dont_call_warnings "
        "because we're only after the RuntimeError"
    ):
        with pytest.raises(
            RuntimeError,
            match=(
                r"The warning triggered on filename \'(.*)test_versions.py\', "
                r"line number ([\d]+), is supposed to be shown until version "
                r"0.17.0 is released. Current version is now "
                r"(.*). Please remove the warning."
            ),
        ):
            salt.utils.versions.warn_until((0, 17), "Foo", _dont_call_warnings=True)

        with pytest.raises(
            RuntimeError,
            match=(
                r"The warning triggered on filename \'(.*)test_versions.py\', "
                r"line number ([\d]+), is supposed to be shown until version "
                r"(.*) is released. Current version is now "
                r"(.*). Please remove the warning."
            ),
        ):
            salt.utils.versions.warn_until(
                "Hydrogen",
                "Foo",
                _dont_call_warnings=True,
                _version_info_=(sys.maxsize, 16, 0),
            )

    with subtests.test("version on the deprecation message gets properly formatted"):
        with warnings.catch_warnings(record=True) as recorded_warnings:
            vrs = salt.version.SaltStackVersion.from_name("Helium")
            salt.utils.versions.warn_until(
                "Helium",
                "Deprecation Message until {version}!",
                _version_info_=(vrs.major - 1, 0),
            )
            assert "Deprecation Message until {}!".format(vrs.formatted_version) == str(
                recorded_warnings[0].message
            )


def test_kwargs_warn_until_warning_raised(subtests):
    # We *always* want *all* warnings thrown on this module
    warnings.filterwarnings("always", "", DeprecationWarning, __name__)

    def raise_warning(**kwargs):
        _version_info_ = kwargs.pop("_version_info_", (0, 16, 0))
        salt.utils.versions.kwargs_warn_until(
            kwargs, (0, 17), _version_info_=_version_info_
        )

    with subtests.test(
        "raise_warning({...}) should show warning until version info is >= (0, 17)"
    ):
        with warnings.catch_warnings(record=True) as recorded_warnings:
            raise_warning(foo=42)  # with a kwarg
            expected = (
                "The following parameter(s) have been deprecated and will be "
                "removed in '0.17.0': 'foo'."
            )
            assert expected == str(recorded_warnings[0].message)

    with subtests.test(
        "With no **kwargs, should not show warning until version info is >= (0, 17)"
    ):
        with warnings.catch_warnings(record=True) as recorded_warnings:
            salt.utils.versions.kwargs_warn_until(
                {}, (0, 17), _version_info_=(0, 16, 0)  # no kwargs
            )
            assert 0 == len(recorded_warnings)

    with subtests.test(
        "Let's set version info to (0, 17), a RuntimeError should be raised "
        "regardless of whether or not we pass any **kwargs."
    ):
        with pytest.raises(
            RuntimeError,
            match=(
                r"The warning triggered on filename \'(.*)test_versions.py\', "
                r"line number ([\d]+), is supposed to be shown until version "
                r"0.17.0 is released. Current version is now 0.17.0. "
                r"Please remove the warning."
            ),
        ):
            raise_warning(_version_info_=(0, 17))  # no kwargs

        with pytest.raises(
            RuntimeError,
            match=(
                r"The warning triggered on filename \'(.*)test_versions.py\', "
                r"line number ([\d]+), is supposed to be shown until version "
                r"0.17.0 is released. Current version is now 0.17.0. "
                r"Please remove the warning."
            ),
        ):
            raise_warning(bar="baz", qux="quux", _version_info_=(0, 17))  # some kwargs


def test_warn_until_date_warning_raised():
    # We *always* want *all* warnings thrown on this module
    warnings.filterwarnings("always", "", DeprecationWarning, __name__)

    _current_date = datetime.date(2000, 1, 1)

    # Test warning with datetime.date instance
    with warnings.catch_warnings(record=True) as recorded_warnings:
        salt.utils.versions.warn_until_date(
            datetime.date(2000, 1, 2),
            "Deprecation Message!",
            _current_date=_current_date,
        )
        assert "Deprecation Message!" == str(recorded_warnings[0].message)

    # Test warning with datetime.datetime instance
    with warnings.catch_warnings(record=True) as recorded_warnings:
        salt.utils.versions.warn_until_date(
            datetime.datetime(2000, 1, 2),
            "Deprecation Message!",
            _current_date=_current_date,
        )
        assert "Deprecation Message!" == str(recorded_warnings[0].message)

    # Test warning with date as a string
    with warnings.catch_warnings(record=True) as recorded_warnings:
        salt.utils.versions.warn_until_date(
            "20000102", "Deprecation Message!", _current_date=_current_date
        )
        assert "Deprecation Message!" == str(recorded_warnings[0].message)

    # the deprecation warning is not issued because we passed
    # _dont_call_warning
    with warnings.catch_warnings(record=True) as recorded_warnings:
        salt.utils.versions.warn_until_date(
            "20000102",
            "Deprecation Message!",
            _dont_call_warnings=True,
            _current_date=_current_date,
        )
        assert len(recorded_warnings) == 0

    # Let's test for RuntimeError raise
    with pytest.raises(
        RuntimeError,
        match=(
            r"Deprecation Message! This warning\(now exception\) triggered on "
            r"filename \'(.*)test_versions.py\', line number ([\d]+), is "
            r"supposed to be shown until ([\d-]+). Today is ([\d-]+). "
            r"Please remove the warning."
        ),
    ):
        salt.utils.versions.warn_until_date("20000101", "Deprecation Message!")

    # Even though we're calling warn_until_date, we pass _dont_call_warnings
    # because we're only after the RuntimeError
    with pytest.raises(
        RuntimeError,
        match=(
            r"Deprecation Message! This warning\(now exception\) triggered on "
            r"filename \'(.*)test_versions.py\', line number ([\d]+), is "
            r"supposed to be shown until ([\d-]+). Today is ([\d-]+). "
            r"Please remove the warning."
        ),
    ):
        salt.utils.versions.warn_until_date(
            "20000101",
            "Deprecation Message!",
            _dont_call_warnings=True,
            _current_date=_current_date,
        )


def test_warn_until_date_bad_strptime_format():
    # We *always* want *all* warnings thrown on this module
    warnings.filterwarnings("always", "", DeprecationWarning, __name__)

    # Let's test for RuntimeError raise
    with pytest.raises(
        ValueError, match="time data '0022' does not match format '%Y%m%d'"
    ):
        salt.utils.versions.warn_until_date("0022", "Deprecation Message!")



"""
    Unit tests for the salt.utils.nacl module
"""
import os

import pytest

import salt.modules.config as config
import salt.utils.files
from tests.support.mock import patch

pytest.importorskip("nacl.public")
pytest.importorskip("nacl.secret")

import salt.utils.nacl as nacl


@pytest.fixture
def configure_loader_modules():
    return {
        nacl: {"__salt__": {"config.get": config.get}},
        config: {"__opts__": {}},
    }


@pytest.fixture(scope="module")
def test_keygen():
    """
    test nacl.keygen function

    Note: nacl.keygen returns base64 encoded values
    """
    ret = nacl.keygen()
    assert "sk" in ret
    assert "pk" in ret
    return ret


def test_fips_mode():
    """
    Nacl pillar doesn't load when fips_mode is True
    """
    opts = {"fips_mode": True}
    with patch("salt.utils.nacl.__opts__", opts, create=True):
        ret = salt.utils.nacl.__virtual__()
        assert ret == (False, "nacl utils not available in FIPS mode")


def test_keygen_sk_file(test_keygen):
    """
    test nacl.keygen function
    with sk_file set
    """
    with pytest.helpers.temp_file("test_keygen_sk_file") as fpath:
        with salt.utils.files.fopen(fpath, "wb") as wfh:
            wfh.write(test_keygen["sk"])

        # test sk_file
        ret = nacl.keygen(sk_file=fpath)
        assert "saved pk_file: {}.pub".format(fpath) == ret
        salt.utils.files.remove(str(fpath) + ".pub")


def test_keygen_keyfile(test_keygen):
    """
    test nacl.keygen function
    with keyfile set
    """
    with pytest.helpers.temp_file("test_keygen_keyfile") as fpath:
        with salt.utils.files.fopen(fpath, "wb") as wfh:
            wfh.write(test_keygen["sk"])

        ret = nacl.keygen(keyfile=fpath)
        assert "saved pk_file: {}.pub".format(fpath) == ret
        salt.utils.files.remove(str(fpath) + ".pub")


def test_enc_keyfile(test_keygen):
    """
    test nacl.enc function
    with keyfile and pk_file set
    """
    with pytest.helpers.temp_file("test_enc_keyfile") as fpath:
        with salt.utils.files.fopen(fpath, "wb") as wfh:
            wfh.write(test_keygen["sk"])
        with salt.utils.files.fopen(str(fpath) + ".pub", "wb") as wfhpub:
            wfhpub.write(test_keygen["pk"])

        kwargs = {
            "opts": {"pki_dir": os.path.dirname(fpath)},
            "keyfile": str(fpath),
            "pk_file": str(fpath) + ".pub",
        }
        ret = nacl.enc("blah", **kwargs)
        assert isinstance(ret, bytes)
        salt.utils.files.remove(str(fpath) + ".pub")


def test_enc_sk_file(test_keygen):
    """
    test nacl.enc function
    with sk_file and pk_file set
    """
    with pytest.helpers.temp_file("test_enc_sk_file") as fpath:
        with salt.utils.files.fopen(fpath, "wb") as wfh:
            wfh.write(test_keygen["sk"])
        with salt.utils.files.fopen(str(fpath) + ".pub", "wb") as wfhpub:
            wfhpub.write(test_keygen["pk"])

        kwargs = {
            "opts": {"pki_dir": os.path.dirname(fpath)},
            "sk_file": str(fpath),
            "pk_file": str(fpath) + ".pub",
        }
        ret = nacl.enc("blah", **kwargs)
        assert isinstance(ret, bytes)
        salt.utils.files.remove(str(fpath) + ".pub")


def test_dec_keyfile(test_keygen):
    """
    test nacl.dec function
    with keyfile and pk_file set
    """
    with pytest.helpers.temp_file("test_dec_keyfile") as fpath:
        with salt.utils.files.fopen(fpath, "wb") as wfh:
            wfh.write(test_keygen["sk"])
        with salt.utils.files.fopen(str(fpath) + ".pub", "wb") as wfhpub:
            wfhpub.write(test_keygen["pk"])

        kwargs = {
            "opts": {"pki_dir": os.path.dirname(fpath)},
            "keyfile": str(fpath),
            "pk_file": str(fpath) + ".pub",
        }

        enc_data = nacl.enc("blah", **kwargs)
        ret = nacl.dec(enc_data, **kwargs)
        assert isinstance(ret, bytes)
        assert ret == b"blah"
        salt.utils.files.remove(str(fpath) + ".pub")


def test_dec_sk_file(test_keygen):
    """
    test nacl.dec function
    with sk_file and pk_file set
    """
    with pytest.helpers.temp_file("test_dec_sk_file") as fpath:
        with salt.utils.files.fopen(fpath, "wb") as wfh:
            wfh.write(test_keygen["sk"])
        with salt.utils.files.fopen(str(fpath) + ".pub", "wb") as wfhpub:
            wfhpub.write(test_keygen["pk"])

        kwargs = {
            "opts": {"pki_dir": os.path.dirname(fpath)},
            "sk_file": str(fpath),
            "pk_file": str(fpath) + ".pub",
        }

        enc_data = nacl.enc("blah", **kwargs)
        ret = nacl.dec(enc_data, **kwargs)
        assert isinstance(ret, bytes)
        assert ret == b"blah"
        salt.utils.files.remove(str(fpath) + ".pub")



"""
tests.pytests.unit.utils.win_dacl.test_get_sid
~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

Test the get_sid function in the win_dacl utility module
"""
import pytest

import salt.utils.win_dacl

# Third-party libs
try:
    import pywintypes
    import win32security

    HAS_WIN32 = True
except ImportError:
    HAS_WIN32 = False

pytestmark = [
    pytest.mark.windows_whitelisted,
    pytest.mark.skip_unless_on_windows,
    pytest.mark.skipif(not HAS_WIN32, reason="Requires Win32 libraries"),
]


@pytest.mark.parametrize(
    "principal",
    (
        "Administrators",  # Normal
        "adMiniStrAtorS",  # Mixed Case
        "S-1-5-32-544",  # String SID
    ),
)
def test_get_sid(principal):
    """
    Validate getting a pysid object with various inputs
    """
    sid_obj = salt.utils.win_dacl.get_sid(principal)
    assert isinstance(sid_obj, pywintypes.SIDType)
    assert win32security.LookupAccountSid(None, sid_obj)[0] == "Administrators"



import re

import salt.modules.jboss7_cli as jboss7_cli
from salt.exceptions import CommandExecutionError
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.mock import patch
from tests.support.unit import TestCase


class CmdMock:
    commands = []
    command_response_func = None  # if you want to test complete response object (with retcode, stdout and stderr)
    cli_commands = []

    default_response = {
        "retcode": 0,
        "stdout": """ {
        "outcome" => "success"
    }""",
        "stderr": "",
    }

    def __init__(self, command_response_func=None):
        self.command_response_func = command_response_func

    def run_all(self, command):
        self.commands.append(command)
        if self.command_response_func is not None:
            return self.command_response_func(command)

        cli_command = self.__get_cli_command(command)
        self.cli_commands.append(cli_command)
        return self.default_response

    @staticmethod
    def __get_cli_command(command):
        command_re = re.compile(r"--command=\"\s*(.+?)\s*\"$", re.DOTALL)
        m = command_re.search(command)  # --command has to be the last argument
        if m:
            cli_command = m.group(1)
            return cli_command
        return None

    def get_last_command(self):
        if len(self.commands) > 0:
            return self.commands[-1]
        else:
            return None

    def get_last_cli_command(self):
        if len(self.cli_commands) > 0:
            return self.cli_commands[-1]
        else:
            return None

    def clear(self):
        self.commands = []
        self.command_response_func = None
        self.cli_commands = []


class JBoss7CliTestCase(TestCase, LoaderModuleMockMixin):
    cmd = CmdMock()
    jboss_config = {
        "cli_path": "/opt/jboss/jboss-eap-6.0.1/bin/jboss-cli.sh",
        "controller": "123.234.345.456:9999",
        "instance_name": "Instance1",
        "cli_user": "jbossadm",
        "cli_password": "jbossadm",
        "status_url": "http://sampleapp.example.com:8080/",
    }

    def setup_loader_modules(self):
        self.cmd = CmdMock()
        self.addCleanup(delattr, self, "cmd")
        return {jboss7_cli: {"__salt__": {"cmd.run_all": self.cmd.run_all}}}

    def test_controller_authentication(self):
        jboss7_cli.run_operation(self.jboss_config, "some cli operation")

        self.assertEqual(
            self.cmd.get_last_command(),
            "/opt/jboss/jboss-eap-6.0.1/bin/jboss-cli.sh --connect"
            ' --controller="123.234.345.456:9999" --user="jbossadm"'
            ' --password="jbossadm" --command="some cli operation"',
        )

    def test_controller_without_authentication(self):
        jboss_config = {
            "cli_path": "/opt/jboss/jboss-eap-6.0.1/bin/jboss-cli.sh",
            "controller": "123.234.345.456:9999",
        }
        jboss7_cli.run_operation(jboss_config, "some cli operation")

        self.assertEqual(
            self.cmd.get_last_command(),
            "/opt/jboss/jboss-eap-6.0.1/bin/jboss-cli.sh --connect"
            ' --controller="123.234.345.456:9999" --command="some cli operation"',
        )

    def test_operation_execution(self):
        operation = r"sample_operation"
        jboss7_cli.run_operation(self.jboss_config, operation)

        self.assertEqual(
            self.cmd.get_last_command(),
            r"/opt/jboss/jboss-eap-6.0.1/bin/jboss-cli.sh --connect"
            r' --controller="123.234.345.456:9999" --user="jbossadm"'
            r' --password="jbossadm" --command="sample_operation"',
        )

    def test_handling_jboss_error(self):
        def command_response(command):
            return {
                "retcode": 1,
                "stdout": r"""{
                       "outcome" => "failed",
                       "failure-description" => "JBAS014807: Management resource '[
                       (\"subsystem\" => \"datasources\"),
                       (\"data-source\" => \"non-existing\")
                    ]' not found",
                        "rolled-back" => true,
                        "response-headers" => {"process-state" => "reload-required"}
                    }
                    """,
                "stderr": "some err",
            }

        self.cmd.command_response_func = command_response

        result = jboss7_cli.run_operation(self.jboss_config, "some cli command")

        self.assertFalse(result["success"])
        self.assertEqual(result["err_code"], "JBAS014807")

    def test_handling_cmd_not_exists(self):
        def command_response(command):
            return {
                "retcode": 127,
                "stdout": """Command not exists""",
                "stderr": "some err",
            }

        self.cmd.command_response_func = command_response

        try:
            jboss7_cli.run_operation(self.jboss_config, "some cli command")
            # should throw an exception
            assert False
        except CommandExecutionError as err:
            self.assertTrue(
                str(err).startswith("Could not execute jboss-cli.sh script")
            )

    def test_handling_other_cmd_error(self):
        def command_response(command):
            return {
                "retcode": 1,
                "stdout": """Command not exists""",
                "stderr": "some err",
            }

        self.cmd.command_response_func = command_response

        try:
            jboss7_cli.run_command(self.jboss_config, "some cli command")
            # should throw an exception
            self.fail("An exception should be thrown")
        except CommandExecutionError as err:
            self.assertTrue(str(err).startswith("Command execution failed"))

    def test_matches_cli_output(self):
        text = """{
            "key1" => "value1"
            "key2" => "value2"
            }
            """

        self.assertTrue(jboss7_cli._is_cli_output(text))

    def test_not_matches_cli_output(self):
        text = """Some error """

        self.assertFalse(jboss7_cli._is_cli_output(text))

    def test_parse_flat_dictionary(self):
        text = """{
            "key1" => "value1"
            "key2" => "value2"
            }"""

        result = jboss7_cli._parse(text)

        self.assertEqual(len(result), 2)
        self.assertEqual(result["key1"], "value1")
        self.assertEqual(result["key2"], "value2")

    def test_parse_nested_dictionary(self):
        text = """{
            "key1" => "value1",
            "key2" => {
                "nested_key1" => "nested_value1"
            }
        }"""

        result = jboss7_cli._parse(text)

        self.assertEqual(len(result), 2)
        self.assertEqual(result["key1"], "value1")
        self.assertEqual(len(result["key2"]), 1)
        self.assertEqual(result["key2"]["nested_key1"], "nested_value1")

    def test_parse_string_after_dict(self):
        text = """{
            "result" => {
                "jta" => true
            },
            "response-headers" => {"process-state" => "reload-required"}
        }"""

        result = jboss7_cli._parse(text)

        self.assertTrue(result["result"]["jta"])
        self.assertEqual(result["response-headers"]["process-state"], "reload-required")

    def test_parse_all_datatypes(self):
        text = """{
            "outcome" => "success",
            "result" => {
                "allocation-retry" => undefined,
                "connection-url" => "jdbc:mysql://localhost:3306/appdb",
                "driver-name" => "mysql",
                "enabled" => false,
                "jta" => true
            },
            "response-headers" => {"process-state" => "reload-required"}
        }"""

        result = jboss7_cli._parse(text)

        self.assertEqual(result["outcome"], "success")
        self.assertIsNone(result["result"]["allocation-retry"])
        self.assertEqual(
            result["result"]["connection-url"], "jdbc:mysql://localhost:3306/appdb"
        )
        self.assertEqual(result["result"]["driver-name"], "mysql")
        self.assertEqual(result["result"]["enabled"], False)
        self.assertTrue(result["result"]["jta"])
        self.assertEqual(result["response-headers"]["process-state"], "reload-required")

    def test_multiline_strings_with_escaped_quotes(self):
        text = r"""{
            "outcome" => "failed",
            "failure-description" => "JBAS014807: Management resource '[
            (\"subsystem\" => \"datasources\"),
            (\"data-source\" => \"asc\")
        ]' not found",
            "rolled-back" => true,
            "response-headers" => {"process-state" => "reload-required"}
        }"""

        result = jboss7_cli._parse(text)

        self.assertEqual(result["outcome"], "failed")
        self.assertTrue(result["rolled-back"])
        self.assertEqual(result["response-headers"]["process-state"], "reload-required")
        self.assertEqual(
            result["failure-description"],
            r"""JBAS014807: Management resource '[
            (\"subsystem\" => \"datasources\"),
            (\"data-source\" => \"asc\")
        ]' not found""",
        )

    def test_handling_double_backslash_in_return_values(self):
        text = r"""{
                 "outcome" => "success",
                 "result" => {
                    "binding-type" => "simple",
                    "value" => "DOMAIN\\foo"
                   }
                }"""

        result = jboss7_cli._parse(text)

        self.assertEqual(result["outcome"], "success")
        self.assertEqual(result["result"]["binding-type"], "simple")
        self.assertEqual(result["result"]["value"], r"DOMAIN\foo")

    def test_numbers_without_quotes(self):
        text = r"""{
                "outcome" => "success",
                "result" => {
                    "min-pool-size" => 1233,
                    "new-connection-sql" => undefined
                }
            }"""

        result = jboss7_cli._parse(text)

        self.assertEqual(result["outcome"], "success")
        self.assertEqual(result["result"]["min-pool-size"], 1233)
        self.assertIsNone(result["result"]["new-connection-sql"])

    def test_all_datasource_properties(self):
        text = r"""{
            "outcome" => "success",
            "result" => {
                "allocation-retry" => undefined,
                "allocation-retry-wait-millis" => undefined,
                "allow-multiple-users" => undefined,
                "background-validation" => undefined,
                "background-validation-millis" => undefined,
                "blocking-timeout-wait-millis" => undefined,
                "check-valid-connection-sql" => undefined,
                "connection-properties" => undefined,
                "connection-url" => "jdbc:mysql:thin:@db.example.com",
                "datasource-class" => undefined,
                "driver-class" => undefined,
                "driver-name" => "mysql",
                "enabled" => true,
                "exception-sorter-class-name" => undefined,
                "exception-sorter-properties" => undefined,
                "flush-strategy" => "FailingConnectionOnly",
                "idle-timeout-minutes" => undefined,
                "jndi-name" => "java:/appDS",
                "jta" => true,
                "max-pool-size" => 20,
                "min-pool-size" => 3,
                "new-connection-sql" => undefined,
                "password" => "Password4321",
                "pool-prefill" => undefined,
                "pool-use-strict-min" => undefined,
                "prepared-statements-cache-size" => undefined,
                "query-timeout" => undefined,
                "reauth-plugin-class-name" => undefined,
                "reauth-plugin-properties" => undefined,
                "security-domain" => undefined,
                "set-tx-query-timeout" => false,
                "share-prepared-statements" => false,
                "spy" => false,
                "stale-connection-checker-class-name" => undefined,
                "stale-connection-checker-properties" => undefined,
                "track-statements" => "NOWARN",
                "transaction-isolation" => undefined,
                "url-delimiter" => undefined,
                "url-selector-strategy-class-name" => undefined,
                "use-ccm" => "true",
                "use-fast-fail" => false,
                "use-java-context" => "false",
                "use-try-lock" => undefined,
                "user-name" => "user1",
                "valid-connection-checker-class-name" => undefined,
                "valid-connection-checker-properties" => undefined,
                "validate-on-match" => false,
                "statistics" => {
                    "jdbc" => undefined,
                    "pool" => undefined
                }
            },
            "response-headers" => {"process-state" => "reload-required"}
        }"""

        result = jboss7_cli._parse(text)

        self.assertEqual(result["outcome"], "success")
        self.assertEqual(result["result"]["max-pool-size"], 20)
        self.assertIsNone(result["result"]["new-connection-sql"])
        self.assertIsNone(result["result"]["url-delimiter"])
        self.assertFalse(result["result"]["validate-on-match"])

    def test_datasource_resource_one_attribute_description(self):
        cli_output = """{
            "outcome" => "success",
            "result" => {
                "description" => "A JDBC data-source configuration",
                "head-comment-allowed" => true,
                "tail-comment-allowed" => true,
                "attributes" => {
                    "connection-url" => {
                        "type" => STRING,
                        "description" => "The JDBC driver connection URL",
                        "expressions-allowed" => true,
                        "nillable" => false,
                        "min-length" => 1L,
                        "max-length" => 2147483647L,
                        "access-type" => "read-write",
                        "storage" => "configuration",
                        "restart-required" => "no-services"
                    }
                },
                "children" => {"connection-properties" => {"description" => "The connection-properties element allows you to pass in arbitrary connection properties to the Driver.connect(url, props) method"}}
            }
        }
        """
        result = jboss7_cli._parse(cli_output)

        self.assertEqual(result["outcome"], "success")
        conn_url_attributes = result["result"]["attributes"]["connection-url"]
        self.assertEqual(conn_url_attributes["type"], "STRING")
        self.assertEqual(
            conn_url_attributes["description"], "The JDBC driver connection URL"
        )
        self.assertTrue(conn_url_attributes["expressions-allowed"])
        self.assertFalse(conn_url_attributes["nillable"])
        self.assertEqual(conn_url_attributes["min-length"], 1)
        self.assertEqual(conn_url_attributes["max-length"], 2147483647)
        self.assertEqual(conn_url_attributes["access-type"], "read-write")
        self.assertEqual(conn_url_attributes["storage"], "configuration")
        self.assertEqual(conn_url_attributes["restart-required"], "no-services")

    def test_datasource_complete_resource_description(self):
        cli_output = """{
            "outcome" => "success",
            "result" => {
                "description" => "A JDBC data-source configuration",
                "head-comment-allowed" => true,
                "tail-comment-allowed" => true,
                "attributes" => {
                    "connection-url" => {
                        "type" => STRING,
                        "description" => "The JDBC driver connection URL",
                        "expressions-allowed" => true,
                        "nillable" => false,
                        "min-length" => 1L,
                        "max-length" => 2147483647L,
                        "access-type" => "read-write",
                        "storage" => "configuration",
                        "restart-required" => "no-services"
                    }
                },
                "children" => {"connection-properties" => {"description" => "The connection-properties element allows you to pass in arbitrary connection properties to the Driver.connect(url, props) method"}}
            }
        }
        """

        result = jboss7_cli._parse(cli_output)

        self.assertEqual(result["outcome"], "success")
        conn_url_attributes = result["result"]["attributes"]["connection-url"]
        self.assertEqual(conn_url_attributes["type"], "STRING")
        self.assertEqual(
            conn_url_attributes["description"], "The JDBC driver connection URL"
        )
        self.assertTrue(conn_url_attributes["expressions-allowed"])
        self.assertFalse(conn_url_attributes["nillable"])
        self.assertEqual(conn_url_attributes["min-length"], 1)
        self.assertEqual(conn_url_attributes["max-length"], 2147483647)
        self.assertEqual(conn_url_attributes["access-type"], "read-write")
        self.assertEqual(conn_url_attributes["storage"], "configuration")
        self.assertEqual(conn_url_attributes["restart-required"], "no-services")

    def test_escaping_operation_with_backslashes_and_quotes(self):
        operation = (
            r'/subsystem=naming/binding="java:/sampleapp/web-module/ldap/username":add(binding-type=simple,'
            r' value="DOMAIN\\\\user")'
        )
        jboss7_cli.run_operation(self.jboss_config, operation)

        self.assertEqual(
            self.cmd.get_last_command(),
            r"/opt/jboss/jboss-eap-6.0.1/bin/jboss-cli.sh --connect"
            r' --controller="123.234.345.456:9999" --user="jbossadm"'
            r' --password="jbossadm"'
            r' --command="/subsystem=naming/binding=\"java:/sampleapp/web-module/ldap/username\":add(binding-type=simple,'
            r' value=\"DOMAIN\\\\\\\\user\")"',
        )

    def test_run_operation_wflyctl_error(self):
        call_cli_ret = {
            "retcode": 1,
            "stdout": '{"failure-description" => "WFLYCTL0234523: ops"}',
        }
        with patch(
            "salt.modules.jboss7_cli._call_cli", return_value=call_cli_ret
        ) as _call_cli:
            ret = jboss7_cli.run_operation(None, "ls", False)
            self.assertEqual(ret["err_code"], "WFLYCTL0234523")

    def test_run_operation_no_code_error(self):
        call_cli_ret = {
            "retcode": 1,
            "stdout": '{"failure-description" => "ERROR234523: ops"}',
        }
        with patch(
            "salt.modules.jboss7_cli._call_cli", return_value=call_cli_ret
        ) as _call_cli:
            ret = jboss7_cli.run_operation(None, "ls", False)
            self.assertEqual(ret["err_code"], "-1")



"""
    :codeauthor: Jayesh Kariya <jayeshk@saltstack.com>

    Test cases for salt.modules.rbenv
"""


import os

import pytest

import salt.modules.rbenv as rbenv
from tests.support.mock import MagicMock, patch


@pytest.fixture
def configure_loader_modules():
    return {rbenv: {}}


def test_install():
    """
    Test for install Rbenv systemwide
    """
    with patch.object(rbenv, "_rbenv_path", return_value=True):
        with patch.object(rbenv, "_install_rbenv", return_value=True):
            with patch.object(rbenv, "_install_ruby_build", return_value=True):
                with patch.object(os.path, "expanduser", return_value="A"):
                    assert rbenv.install()


def test_update():
    """
    Test for updates the current versions of Rbenv and Ruby-Build
    """
    with patch.object(rbenv, "_rbenv_path", return_value=True):
        with patch.object(rbenv, "_update_rbenv", return_value=True):
            with patch.object(rbenv, "_update_ruby_build", return_value=True):
                with patch.object(os.path, "expanduser", return_value="A"):
                    assert rbenv.update()


def test_is_installed():
    """
    Test for check if Rbenv is installed.
    """
    with patch.object(rbenv, "_rbenv_bin", return_value="A"):
        with patch.dict(rbenv.__salt__, {"cmd.has_exec": MagicMock(return_value=True)}):
            assert rbenv.is_installed()


def test_install_ruby():
    """
    Test for install a ruby implementation.
    """
    with patch.dict(rbenv.__grains__, {"os": "FreeBSD"}):
        with patch.dict(rbenv.__salt__, {"config.get": MagicMock(return_value="True")}):
            with patch.object(
                rbenv,
                "_rbenv_exec",
                return_value={"retcode": 0, "stderr": "stderr"},
            ):
                with patch.object(rbenv, "rehash", return_value=None):
                    assert rbenv.install_ruby("ruby") == "stderr"

            with patch.object(
                rbenv,
                "_rbenv_exec",
                return_value={"retcode": 1, "stderr": "stderr"},
            ):
                with patch.object(rbenv, "uninstall_ruby", return_value=None):
                    assert not rbenv.install_ruby("ruby")


def test_uninstall_ruby():
    """
    Test for uninstall a ruby implementation.
    """
    with patch.object(rbenv, "_rbenv_exec", return_value=None):
        assert rbenv.uninstall_ruby("ruby", "runas")


def test_versions():
    """
    Test for list the installed versions of ruby.
    """
    with patch.object(rbenv, "_rbenv_exec", return_value="A\nBC\nD"):
        assert rbenv.versions() == ["A", "BC", "D"]


def test_default():
    """
    Test for returns or sets the currently defined default ruby.
    """
    with patch.object(rbenv, "_rbenv_exec", MagicMock(side_effect=[None, False])):
        assert rbenv.default("ruby", "runas")

        assert rbenv.default() == ""


def test_list_():
    """
    Test for list the installable versions of ruby.
    """
    with patch.object(rbenv, "_rbenv_exec", return_value="A\nB\nCD\n"):
        assert rbenv.list_() == ["A", "B", "CD"]


def test_rehash():
    """
    Test for run rbenv rehash to update the installed shims.
    """
    with patch.object(rbenv, "_rbenv_exec", return_value=None):
        assert rbenv.rehash()


def test_do_with_ruby():
    """
    Test for execute a ruby command with rbenv's shims using a
    specific ruby version.
    """
    with patch.object(rbenv, "do", return_value="A"):
        assert rbenv.do_with_ruby("ruby", "cmdline") == "A"
